/* -----------------------------------------

   Create deltaP x-tables for all available values of alphas

   ----------------------------------------- */
#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <cmath>
#include <ctime>
#include <cstdlib>
#include <sys/time.h> 

#include "hell.hh"


using namespace std;


string sas(double as) {
  ostringstream os;
  int ias = int(1000*as);
  if(1000*as-ias>0.5) ias += 1;
  if     (as<0.01) os << "000" << ias;
  else if(as<0.1 ) os << "00"  << ias;
  else if(as<1.  ) os << "0"   << ias;
  else os << ias;
  return os.str();
}


int main (int argc, char* argv[]) {

  struct timeval t0, t1;
  gettimeofday(&t0,NULL);

  bool nll = false;
  if(argc>1) nll = true;
  HELL::LogOrder ord = HELL::LL;
  if(nll) ord = HELL::NLL;


  // x grid
  int Np1 = 300, Np2 = 20-1;
  double x;
  double x_min = 1e-9;
  double x_mid = 0.1;
  double x_max = 0.99+0.01;

  for(int nf=3; nf<=6; nf++) {
    HELL::HELLnf sxD(nf, ord, "./data/");
    sxD.SetLargexDamping();
    sxD.SetMomentumConservation();
    vector<double> as1, as2;
    sxD.GetAvailableAlphas(as1, as2);
    // alphas grid
    double as_min = as1[0];
    double as_max = as1[as1.size()-1];
    if(nll) {
      if(as2[0]>as_min) as_min = as2[0];
      if(as2[as2.size()-1]<as_max) as_max = as2[as2.size()-1];
    }
    int Nas = (as_max-as_min)*1000 +1;
    Nas = (as_max-as_min)*100 +1; // smaller set; comment out for a (10 times) larger set
    //
    for(int a=0; a<Nas; a++) {
      double as = as_min + a*(as_max-as_min)/(Nas-1.);
      //as = int(1000*as)/1000.;  // useless now
      //cout << setprecision(10) << 1000*as << "  " << sas(as) << endl;
      string sord = (nll ? "nlo" : "lo");
      ostringstream filename;
      filename << "./data/x-space/xtable_" << sord << "_nf" << nf << "_alphas" << sas(as) << ".table";
      ofstream ofile(filename.str());
      ofile
	<< setw(15) << Np1
	<< setw(15) << Np2
	<< setw(15) << x_min
	<< setw(15) << x_mid
	<< setw(15) << x_max
	<< endl;
      //
      for(int i=0; i<Np1+Np2; i++) {
	if(i<Np1) x = x_min * exp(i/(Np1-1.)*log(x_mid/x_min));
	else      x = x_mid + (i-Np1+1)*(x_max-x_mid)/(Np2-0.);
	//cout << x << endl;
	HELL::sqmatrix<double> dP = sxD.DeltaP(as, x);
	double dPgg = x*dP.gg();
	double dPqg = x*dP.qg();
	ofile
	  //<< setw(15) << x
	  << setw(15) << dPgg;
	if(nll) ofile << setw(15) << dPqg;
	ofile << endl;
      }
      cout << ".";
      cout.flush();
      //
      ofile.close();
    }
    cout << endl;
  }


  // Finish time
  gettimeofday(&t1,NULL);
  double t=t1.tv_sec-t0.tv_sec+(t1.tv_usec-t0.tv_usec)*0.000001;
  cout << "Total time: " << t << "s" << endl;

  return 0;

}
