#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>
#include <gsl/gsl_errno.h>

#include "CoefficientFunctionsOffShell.hh"


using namespace std;



namespace HELL {

  //const double CA = 3.;
  //const double CF = 4./3.;

  /// off-shell cross sections
  inline double Power(double a, double b) { return pow(a,b); }
  inline double Sqrt(double a) { return sqrt(a); }
  inline double Log(double a) { return log(a); }
  inline double ArcSinh(double a) { return log(a+sqrt(1+a*a)); }
  inline double ArcCsch(double a) { return log(1/a+sqrt(1+1/a/a)); }
  inline double ArcCoth(double a) { return (log(1+1./a)-log(1-1./a))/2.; }
  inline double ArcTanh(double a) { return (log(1+a)-log(1-a))/2.; }
  //double PolyLog(int a, double b) {
  //  double res;
  //  if     (a==2) res = li2(b);
  //  else if(a==3) res = li3(b);
  //  else if(a==4) res = li4(b);
  //  else abort();
  //  return res;
  //}
  double Dxi_CLoff(double xi);
  double Dxi_CLoff(double xi, double y) {
    if(xi > 1.e14) return 0;
    if(xi == 0) return 1;
    //double res = -(3*(-1 + Power(y,2))*(-3*Sqrt(xi*(1 + xi - Power(y,2))) + 3*Power(y,2)*Sqrt(xi*(1 + xi - Power(y,2))) - 2*Sqrt(Power(xi,3)*(1 + xi - Power(y,2))) + (-1 + Power(y,2))*(-3 - 4*xi + 3*Power(y,2))*ArcCsch(Sqrt((1 - Power(y,2))/xi))))/(4.*Power(xi,2.5)*Power(1 + xi - Power(y,2),1.5));
    double res = (3*(-1 + Power(y,2))*(3*Sqrt(xi*(1 + xi - Power(y,2))) - 3*Power(y,2)*Sqrt(xi*(1 + xi - Power(y,2))) + 2*Sqrt(Power(xi,3)*(1 + xi - Power(y,2))) + (3 + 4*xi - 3*Power(y,2))*(-1 + Power(y,2))*ArcCsch(Sqrt((1 - Power(y,2))/xi))))/(4.*Power(xi,2.5)*Power(1 + xi - Power(y,2),1.5));
    //double res = -3./8. * (y/(1+xi*y)/(1+xi*y) + 1./(y+xi)/(y+xi)) * (-3./4.*(1+y)/y + 1./8./y/sqrt(y)*(3+2*y+3*y*y)*log((1+sqrt(y))/(1-sqrt(y))));
    //cout << res << endl;
    return res;
  }
  double Dxi_C2off(double xi);
  double Dxi_C2off(double xi, double y) {
    if(xi > 1.e14) return 0;
    if(xi == 0) return 0;
    double res = -(y*(-2 + xi - 2*(1 + 2*xi)*Power(y,2) + 3*xi*Power(y,4))*(-1 + xi*(-1 + Power(y,2))) - Power(xi,1.5)*y*Sqrt(1 + 1/xi - Power(y,2))*Power(-1 + Power(y,2),2)*(-4 + xi*(-1 + 3*Power(y,2)))*ArcCsch(Sqrt(xi - xi*Power(y,2))))/(2.*xi*y*Power(1 + xi - xi*Power(y,2),2));
    return res*3./4.; /// check this!!!
  }

  double ap6[6] = {-1.07455, 1633.33, -38391.2, 199172., -343462., 185294.};
  double ap7[7] = {-58.32701753612378,23618.27574514618,-664446.8966666108,4.889294623552245e6,-1.376200860973154e7,1.6212320660971168e7,-6.735695559155129e6};
  double Dxi_Hqg(double xi) {
    if(xi>1) return 0;
    double sum = 0;
    for(int p=0; p<7; p++)
      sum += ap7[p] * pow(xi,p) * (p+1);
    return sum;
  }


const unsigned C2_N_POINTS = 500;
const double C2_H = double(1)/C2_N_POINTS;

const double C2_TABLE[C2_N_POINTS+1] = {0,
                                      2.8813969329189988142390401578,
                                      2.8753964167265948213731376597,
                                      2.86996479964919465788939116938,
                                      2.86490534412739164631791350543,
                                      2.86012436893071264845576472524,
                                      2.85556655605311741165283344153,
                                      2.85119526022841907864597998331,
                                      2.84698436990877117364559001385,
                                      2.84291431177045012387351503597,
                                      2.83896985427178937049764786289,
                                      2.83513879834284690069916124658,
                                      2.83141114768481059818817575091,
                                      2.82777855717348610969908975342,
                                      2.82423395169241298314938757281,
                                      2.82077125420326042023845126346,
                                      2.81738518648517623226993649257,
                                      2.81407111975635753823544291171,
                                      2.81082496046696764246776676496,
                                      2.80764306147317179685313633831,
                                      2.80452215190263832180099396861,
                                      2.80145928103353074391362233688,
                                      2.79845177284818128915716147414,
                                      2.79549718883464133124354211585,
                                      2.79259329724315442297436214104,
                                      2.78973804745324351516952248393,
                                      2.78692954842994095788202746891,
                                      2.78416605048350204094472358768,
                                      2.78144592972155778463283420773,
                                      2.77876767471359011493480676774,
                                      2.77612987498691871541314111063,
                                      2.7735312110495146209926172162,
                                      2.77097044569388804551345261084,
                                      2.76844641638233824238288770948,
                                      2.76595802855012837523635443713,
                                      2.7635042496919569707020916304,
                                      2.76108410412014660456375068539,
                                      2.75869666830154008216656119668,
                                      2.7563410666951530495894194207,
                                      2.75401646802491877626854280607,
                                      2.75172208193194424791028746546,
                                      2.74945715595901724119201556743,
                                      2.74722097282700554927728781104,
                                      2.74501284796854214394880662133,
                                      2.74283212728920741300549563524,
                                      2.74067818513047163147269857006,
                                      2.73855042241208346625239891597,
                                      2.73644826493449293981007303419,
                                      2.73437116182436831995783109399,
                                      2.73231858410837771781426726356,
                                      2.73029002340221643167770408503,
                                      2.7282849907034183844952002925,
                                      2.7263030152778339957497782233,
                                      2.72434364363082028017315495658,
                                      2.72240643855519908991885074099,
                                      2.72049097824891889618388880778,
                                      2.71859685549612331502829512008,
                                      2.71672367690600164818609029939,
                                      2.71487106220438644857933304775,
                                      2.71303864357358186819990305333,
                                      2.71122606503636391193429312927,
                                      2.70943298188049787410675426391,
                                      2.70765906012047614055150287734,
                                      2.70590397599349715364508060129,
                                      2.70416741548698876702743142712,
                                      2.70244907389523084978091547075,
                                      2.70074865540285661886147430584,
                                      2.69906587269321305084876104194,
                                      2.6974004465797406786663608227,
                                      2.69575210565869457105247505208,
                                      2.69412058598167345775291847039,
                                      2.69250563074655466308172718277,
                                      2.69090699000555037052927265692,
                                      2.6893244203892071845038586219,
                                      2.68775768484526722996512667928,
                                      2.68620655239139623305730379157,
                                      2.68467079788086312097418647294,
                                      2.6831502017803275182214897179,
                                      2.68164454995895685095158331812,
                                      2.68015363348815426633411228096,
                                      2.6786772484512328195747857706,
                                      2.67721519576242090060046446402,
                                      2.67576728099462913096142389838,
                                      2.67433331421545037356891095402,
                                      2.67291310983090243306274108821,
                                      2.67150648643645781295984828372,
                                      2.6701132666749368324751464597,
                                      2.6687332771008697554169130885,
                                      2.66736634805096058294348535726,
                                      2.6660123135203100241413684253,
                                      2.66467101104407807478615104948,
                                      2.66334228158428777762195059361,
                                      2.66202596942149126238739238688,
                                      2.66072192205103721081527397443,
                                      2.65942999008369558759828641766,
                                      2.65815002715041093741030008074,
                                      2.65688188981096987424451236907,
                                      2.65562543746638167662081323865,
                                      2.6543805322747832359691485838,
                                      2.65314703907069106324286131339,
                                      2.6519248252874337110582614411,
                                      2.65071376088260787957029928506,
                                      2.64951371826641070235432969758,
                                      2.64832457223270930710217273371,
                                      2.64714619989271676364325548465,
                                      2.64597848061115101315504589413,
                                      2.64482129594476035811696499874,
                                      2.64367452958310561983107681727,
                                      2.64253806729149517330154786523,
                                      2.64141179685597477921978344759,
                                      2.64029560803027947845543227896,
                                      2.63918939248465982217618342022,
                                      2.63809304375649940475181440795,
                                      2.63700645720264506923964534011,
                                      2.63592952995337528702907229231,
                                      2.63486216086793609305696576301,
                                      2.63380425049157760333333727103,
                                      2.63275570101402756842963344726,
                                      2.63171641622934163993895709644,
                                      2.63068630149707306044809888309,
                                      2.62966526370470734396076021692,
                                      2.62865321123131020472657761901,
                                      2.62765005391233952893804098591,
                                      2.62665570300557457583447096988,
                                      2.62567007115811785174181511253,
                                      2.62469307237442723114670419871,
                                      2.62372462198533791109903942906,
                                      2.62276463661803568653308047743,
                                      2.62181303416694483143888155443,
                                      2.62086973376549557066393925646,
                                      2.61993465575873773549026648534,
                                      2.61900772167676871861162368264,
                                      2.61808885420894528594331770943,
                                      2.61717797717885016869288330253,
                                      2.61627501551998565383674373939,
                                      2.61537989525216761881519528491,
                                      2.61449254345859462082467166162,
                                      2.61361288826356775624099242304,
                                      2.61274085881083805489774945494,
                                      2.61187638524255917039391600751,
                                      2.61101939867882407433020181214,
                                      2.61016983119776536219769194041,
                                      2.609327615816199634208633727,
                                      2.60849268647079722814486619682,
                                      2.60766497799975935562618092,
                                      2.6068444261249854302463249864,
                                      2.60603096743471407783139108161,
                                      2.60522453936662198756166918545,
                                      2.60442508019136539966453012598,
                                      2.60363252899654963252449383394,
                                      2.6028468256711126309575582223,
                                      2.6020679108901090695554652145,
                                      2.60129572609988207182853953711,
                                      2.60053021350361010868688721959,
                                      2.59977131604721711984546338599,
                                      2.59901897740563436019274594625,
                                      2.59827314196940291113162462596,
                                      2.59753375483160621542731047002,
                                      2.59680076177512239416382731556,
                                      2.59607410926018648694550811774,
                                      2.59535374441225312235822392963,
                                      2.59463961501015047575320304554,
                                      2.59393166947451670641470377852,
                                      2.59322985685651038685883081181,
                                      2.59253412682678674408130187432,
                                      2.59184442966473182668583087461,
                                      2.59116071624794699360512384345,
                                      2.59048293804197639016283154763,
                                      2.5898110470902703360751337191,
                                      2.58914499600437779818320159497,
                                      2.58848473795436135874289550523,
                                      2.58783022665942831844967761309,
                                      2.58718141637877179249404105779,
                                      2.58653826190261586825160392453,
                                      2.58590071854345909511620526387,
                                      2.5852687421275107708669326592,
                                      2.58464228898631467518449481644,
                                      2.58402131594855507984374954054,
                                      2.58340578033204003703509960799,
                                      2.58279563993585711251901413516,
                                      2.58219085303269688919071872123,
                                      2.58159137836133971940704043675,
                                      2.58099717511930135137153598445,
                                      2.58040820295563319624129962232,
                                      2.57982442196387313865076542301,
                                      2.57924579267514292427417755397,
                                      2.57867227605138828408789242088,
                                      2.57810383347875807635447983329,
                                      2.57754042676111884423094002201,
                                      2.5769820181137012994920599832,
                                      2.57642857015687535133688011957,
                                      2.57588004591005040378285488056,
                                      2.57533640878569774591197442205,
                                      2.57479762258349195637166986846,
                                      2.57426365148456833719934228898,
                                      2.57373446004589348237458244256,
                                      2.57321001319474617364283876329,
                                      2.57269027622330588022753502631,
                                      2.57217521478334622017767383701,
                                      2.57166479488103081940246569781,
                                      2.57115898287180908003589281481,
                                      2.57065774545540944275973011566,
                                      2.57016104967092779819601070418,
                                      2.56966886289200877055731091296,
                                      2.5691811528221176625092992613,
                                      2.56869788748990091374439848724,
                                      2.56821903524463298717390032825,
                                      2.56774456475174765600048261871,
                                      2.56727444498845172231230352696,
                                      2.56680864523941925331881560436,
                                      2.56634713509256447499907144099,
                                      2.56588988443489151482444505827,
                                      2.56543686344841923541531788577,
                                      2.56498804260617944955854817401,
                                      2.56454339266828685400998414444,
                                      2.56410288467807906499189636024,
                                      2.56366648995832518232458876269,
                                      2.56323418010750135175789908479,
                                      2.56280592699613183634292506196,
                                      2.56238170276319414765612861997,
                                      2.56196147981258682640399497907,
                                      2.56154523080965849944176980731,
                                      2.56113292867779687657775679638,
                                      2.56072454659507638574678256905,
                                      2.5603200579909631792626160106,
                                      2.55991943654307627693766708852,
                                      2.55952265617400364392597000937,
                                      2.55912969104817203223760977834,
                                      2.55874051556876944502332457767,
                                      2.55835510437471911196962609661,
                                      2.55797343233770389250877334647,
                                      2.55759547455924005106444480592,
                                      2.55722120636779937525194832293,
                                      2.55685060331597863385914430215,
                                      2.55648364117771539657772590444,
                                      2.55612029594554926185987038172,
                                      2.55576054382792756296734100474,
                                      2.55540436124655464528273579713,
                                      2.55505172483378383028870690376,
                                      2.55470261143005120331271269829,
                                      2.55435699808135038320348953645,
                                      2.55401486203674745257142738748,
                                      2.55367618074593524710813208041,
                                      2.55334093185682622181965895636,
                                      2.55300909321318313078151406433,
                                      2.55268064285228677526917712238,
                                      2.55235555900264009285260149959,
                                      2.5520338200817078772832712994,
                                      2.5517154046936914357637344309,
                                      2.55140029162733750648730650441,
                                      2.5510884598537807751845322135,
                                      2.55077988852441934482715446461,
                                      2.5504745569688225276334306197,
                                      2.55017244469267034310382115303,
                                      2.54987353137572412000606662748,
                                      2.54957779686982761403572661224,
                                      2.54928522119693806631421336509,
                                      2.54899578454718664096263667635,
                                      2.54870946727696769271741149282,
                                      2.54842624990705632794321368282,
                                      2.54814611312075373446078259862,
                                      2.54786903776205976735119108387,
                                      2.54759500483387228933412488244,
                                      2.54732399549621277545469894247,
                                      2.54705599106447770266033935151,
                                      2.54679097300771525541492639939,
                                      2.54652892294692688879008597833,
                                      2.54626982265339330050131307039,
                                      2.5460136540470243731273231364,
                                      2.54576039919473265727220627789,
                                      2.54551004030882997570890689169,
                                      2.54526255974544673758632946173,
                                      2.54501794000297356059780870817,
                                      2.54477616372052480760238583883,
                                      2.54453721367642365256869325562,
                                      2.54430107278670829888045150286,
                                      2.54406772410365898100860833546,
                                      2.5438371508143453883237916238,
                                      2.54360933623919415739961064555,
                                      2.54338426383057608654785016539,
                                      2.54316191717141273353601173804,
                                      2.54294227997380206447105330896,
                                      2.54272533607766282869548698636,
                                      2.54251106944939734123798615883,
                                      2.54229946418057236089494749109,
                                      2.54209050448661775839652674358,
                                      2.541884174705542675334856289,
                                      2.54168045929666888060765839636,
                                      2.54147934283938103706136351639,
                                      2.54128081003189359680807304302,
                                      2.54108484569003404934409617144,
                                      2.54089143474604225211804820921,
                                      2.5407005622473855785872175497,
                                      2.54051221335558962406557569494,
                                      2.54032637334508421480879879634,
                                      2.54014302760206447080426767521,
                                      2.53996216162336667764039505397,
                                      2.53978376101535872762287728769,
                                      2.53960781149284489498857457964,
                                      2.5394342988779847146435907299,
                                      2.53926320909922573832356697911,
                                      2.5390945281902499464439572291,
                                      2.53892824228893359817876602839,
                                      2.53876433763632030648048044578,
                                      2.53860280057560712883421116701,
                                      2.53844361755114346852780274747,
                                      2.53828677510744258511923327209,
                                      2.53813225988820551659528675527,
                                      2.53798005863535721944346840988,
                                      2.537830158188094736504600407,
                                      2.5376825454819472060385789978,
                                      2.53753720754784752892242799114,
                                      2.53739413151121551431002565018,
                                      2.53725330459105232741863296458,
                                      2.53711471409904606637047742414,
                                      2.53697834743868829820996160574,
                                      2.53684419210440138734033277535,
                                      2.53671223568067645267958152876,
                                      2.53658246584122179282559956084,
                                      2.5364548703481216214468378519,
                                      2.53632943705100495797844079014,
                                      2.53620615388622452150661931517,
                                      2.53608500887604547846735511688,
                                      2.53596599012784389747084534811,
                                      2.5358490858333147671918105941,
                                      2.53573428426768943583926688125,
                                      2.53562157378896233323893687252,
                                      2.53551094283712683902844146669,
                                      2.53540237993342016288103107612,
                                      2.53529587367957710503911214496,
                                      2.53519141275709256775539221071,
                                      2.53508898592649269050826720256,
                                      2.53498858202661448408023783133,
                                      2.5348901899738938407647678312,
                                      2.53479379876166180009915618293,
                                      2.5346993974594489516097306104,
                                      2.53460697521229785810199537622,
                                      2.5345165212400833850332707438,
                                      2.5344280248368408234698064993,
                                      2.53434147537010169605527451715,
                                      2.53425686228023713730385791697,
                                      2.53417417507980874137974555708,
                                      2.53409340335292677233657602847,
                                      2.5340145367546156335660981422,
                                      2.53393756501018649494584657455,
                                      2.53386247791461697788177214764,
                                      2.53378926533193780011429595126,
                                      2.53371791719462628379593498502,
                                      2.53364842350300663195621467532,
                                      2.53358077432465688004576212992,
                                      2.53351495979382243079696667403,
                                      2.53345097011083608215408662722,
                                      2.53338879554154445951184171646,
                                      2.53332842641674076495901046934,
                                      2.5332698531316037576529865641,
                                      2.53321306614514288085325673029,
                                      2.5331580559796494525169492714,
                                      2.53310481322015383770855549364,
                                      2.53305332851388852239922056388,
                                      2.53300359256975700952919569179,
                                      2.53295559615780845948068635049,
                                      2.53290933010871799835795441566,
                                      2.53286478531327261869765546421,
                                      2.53282195272186259843552318737,
                                      2.53278082334397836513614574715,
                                      2.53274138824771273365119673436,
                                      2.53270363855926844650855727464,
                                      2.53266756546247094745075550805,
                                      2.53263316019828631963650379865,
                                      2.53260041406434432109427050302,
                                      2.53256931841446645107220934639,
                                      2.53253986465819898196480262192,
                                      2.53251204426035089251366180894,
                                      2.53248584874053663897846839608,
                                      2.53246126967272370195441687513,
                                      2.53243829868478484747512005577,
                                      2.53241692745805504198512412694,
                                      2.53239714772689296169431865472,
                                      2.53237895127824703773796788848,
                                      2.53236232995122597946117903506,
                                      2.53234727563667371902569721119,
                                      2.53233378027674872140030443272,
                                      2.53232183586450760464412246187,
                                      2.53231143444349301622509040411,
                                      2.53230256810732571193411418501,
                                      2.53229522899930078475916596139,
                                      2.53228940931198799187323977671,
                                      2.53228510128683612866583131916,
                                      2.53228229721378139950978391697,
                                      2.53228098943085973570420299418,
                                      2.53228117032382301176995399762,
                                      2.53228283232575911199728514492,
                                      2.53228596791671579985561119706,
                                      2.53229056962332834357370705572,
                                      2.53229663001845085188473395777,
                                      2.53230414172079127460489456565,
                                      2.53231309739455002337731919332,
                                      2.53232348974906216856425143705,
                                      2.53233531153844316891095021551,
                                      2.53234855556123809123417434816,
                                      2.5323632146600742780068781884,
                                      2.53237928172131742131903065935,
                                      2.53239674967473100229247891911,
                                      2.53241561149313905561571094727,
                                      2.53243586019209221944242337827,
                                      2.53245748882953703146616244099,
                                      2.53248049050548843254216328513,
                                      2.53250485836170543977704862041,
                                      2.53253058558136995154743985723,
                                      2.5325576653887686474399573531,
                                      2.53258609104897794662771171553,
                                      2.53261585586755198871238249511,
                                      2.53264695319021360156650754673,
                                      2.53267937640254822120782586843,
                                      2.53271311892970072922658546009,
                                      2.53274817423607517376779895845,
                                      2.53278453582503734054365352733,
                                      2.53282219723862014081680456297,
                                      2.53286115205723178375324896271,
                                      2.53290139389936670099402371898,
                                      2.53294291642131919173824720338,
                                      2.53298571331689975706614856857,
                                      2.53302977831715409265984727237,
                                      2.53307510519008470950187911447,
                                      2.53312168774037515254694397708,
                                      2.53316951980911678777119765346,
                                      2.5332185952735381284057471393,
                                      2.5332689080467366715569544526,
                                      2.53332045207741321680582488407,
                                      2.53337322134960863876226561653,
                                      2.53342720988244308592746160129,
                                      2.53348241172985757858913686804,
                                      2.53353882098035797884015826508,
                                      2.53359643175676130617089998727,
                                      2.53365523821594437244012402385,
                                      2.53371523454859471037794463656,
                                      2.53377641497896377011783291027,
                                      2.53383877376462235859267706014,
                                      2.53390230519621829696274034218,
                                      2.533967003597236271571044001,
                                      2.53403286332375985424433874546,
                                      2.53409987876423566807550399068,
                                      2.53416804433923967513601698978,
                                      2.53423735450124556287514970245,
                                      2.53430780373439520626586381417,
                                      2.53437938655427118305606606664,
                                      2.53445209750767131977803769915,
                                      2.53452593117238524645854245211,
                                      2.53460088215697293825742481088,
                                      2.53467694510054522254351001307,
                                      2.53475411467254623019338435708,
                                      2.53483238557253777017124163011,
                                      2.5349117525299856067165006923,
                                      2.53499221030404761873040068588,
                                      2.5350737536833638212133328994,
                                      2.53515637748584822886133958628,
                                      2.5352400765584825421830662875,
                                      2.53532484577711163674756043887,
                                      2.53541068004624083641872899869,
                                      2.53549757429883495167406403755,
                                      2.53558552349611906434347901402,
                                      2.53567452262738104033882997436,
                                      2.53576456670977575217598416921,
                                      2.53585565078813099331920146337,
                                      2.53594776993475506660216821322,
                                      2.53604091924924602920132471465,
                                      2.53613509385830257685521054201,
                                      2.53623028891553655023847073677,
                                      2.53632649960128704661097248477,
                                      2.53642372112243612007122827629,
                                      2.53652194871222605394905824278,
                                      2.5366211776300781890752011258,
                                      2.53672140316141329186544895358,
                                      2.53682262061747344635388286156,
                                      2.53692482533514545450397359428,
                                      2.5370280126767857293177261914,
                                      2.53713217803004666545173946462,
                                      2.53723731680770447223506155314,
                                      2.53734342444748845416709672318,
                                      2.53745049641191172415459847219,
                                      2.53755852818810333492501194366,
                                      2.53766751528764181422914592201,
                                      2.53777745324639008961940176987,
                                      2.53788833762433178876060336824,
                                      2.53800016400540890139889747574,
                                      2.53811292799736078928026628071,
                                      2.53822662523156453047395092941,
                                      2.53834125136287658471756344589,
                                      2.53845680206947576655990101863,
                                      2.53857327305270751323450676729,
                                      2.53869066003692943435187983147,
                                      2.5388089587693581306509593283,
                                      2.53892816501991726920112517593,
                                      2.53904827458108690259450714568,
                                      2.53916928326775401981490436313,
                                      2.53929118691706431661412282952,
                                      2.5394139813882751733690698096,
                                      2.53953766256260982853353200966,
                                      2.53966222634311273593723967985,
                                      2.53978766865450609432161091656,
                                      2.53991398544304753763650878733};

double DC2_TABLE[C2_N_POINTS+1] = {0,
                                 0.097658059397687506881360457,
                                 0.097513717136168501736890249,
                                 0.097348944685856864963729039,
                                 0.097166016047687981520669144,
                                 0.096966540025739649276952434,
                                 0.096751770324139150528440083,
                                 0.0965227374472385870033022752,
                                 0.0962803178401473514367804886,
                                 0.0960252748472897465910334552,
                                 0.0957582850490984129174990257,
                                 0.0954799562306984846001616061,
                                 0.0951908402060249829632079145,
                                 0.0948914423006558279919473489,
                                 0.0945822285679996562402983661,
                                 0.0942636314123868363200305223,
                                 0.0939360540590106199658439894,
                                 0.0935998741681534392358177144,
                                 0.0932554468007464911179744420,
                                 0.0929031068830558251765414206,
                                 0.0925431712783187040662303850,
                                 0.0921759405455147809527924580,
                                 0.0918017004459213560772647915,
                                 0.0914207232440241508850698980,
                                 0.0910332688390327955172452554,
                                 0.0906395857555629171985139126,
                                 0.0902399120162405928739141635,
                                 0.0898344759145432162828590110,
                                 0.0894234967027530233484875614,
                                 0.0890071852072101218321527336,
                                 0.0885857443809271643061622729,
                                 0.0881593698019339172736927299,
                                 0.0877282501243581630009846930,
                                 0.0872925674881459496645197623,
                                 0.0868524978924236325231391895,
                                 0.0864082115367641323217865581,
                                 0.0859598731340078825137194558,
                                 0.0855076421977798633367346331,
                                 0.0850516733074182584328129390,
                                 0.0845921163526721377742293954,
                                 0.0841291167602228989427762078,
                                 0.0836628157038271968599365076,
                                 0.0831933502996598987947020332,
                                 0.0827208537882478672975352573,
                                 0.0822454557042239409512602846,
                                 0.0817672820349911395501545462,
                                 0.0812864553692664093921022358,
                                 0.0808030950363682944869589484,
                                 0.0803173172370214081581596827,
                                 0.0798292351663705206736417844,
                                 0.0793389591298268291565210191,
                                 0.0788465966523071575275945640,
                                 0.0783522525813722855145131925,
                                 0.0778560291847223452909750617,
                                 0.0773580262424644194303395371,
                                 0.0768583411345294158358178815,
                                 0.0763570689235813789308515374,
                                 0.0758543024337321032395180117,
                                 0.0753501323253468001576489725,
                                 0.0748446471662022474434688967,
                                 0.0743379334992369921624846357,
                                 0.0738300759071134942513354872,
                                 0.0733211570737943400248987586,
                                 0.0728112578433186056029569787,
                                 0.0723004572759499197727602193,
                                 0.0717888327018545983615923060,
                                 0.0712764597724562523366026486,
                                 0.0707634125096023817884603030,
                                 0.0702497633526685451687093471,
                                 0.0697355832037166383577444110,
                                 0.0692209414708155435717982391,
                                 0.0687059061096248360322618851,
                                 0.0681905436633352977331215408,
                                 0.06767491930105362122552353879,
                                 0.06715909685471283747946620714,
                                 0.06664313885458462186480928515,
                                 0.06612710656346467758898942639,
                                 0.06561106000959782758245973131,
                                 0.06509505801840522891923806625,
                                 0.06457915824307222705616782088,
                                 0.06406341719405176229350427841,
                                 0.06354789026753490253395872164,
                                 0.06303263177293698177782341625,
                                 0.06251769495944495219252465546,
                                 0.06200313204166889037305427963,
                                 0.06148899422443811867302939678,
                                 0.06097533172678009492587684700,
                                 0.06046219380511807460106136221,
                                 0.05994962877572154583214207665,
                                 0.05943768403644156833426149556,
                                 0.05892640608776140154748173416,
                                 0.05841584055319117587233093380,
                                 0.05790603219903383491109360768,
                                 0.05739702495354814825053053616,
                                 0.05688886192553325625574264425,
                                 0.05638158542235795394200629364,
                                 0.05587523696745674415911285968,
                                 0.05536985731731358547287719781,
                                 0.05486548647795322213057220632,
                                 0.05436216372095900763646056746,
                                 0.05385992759903521540386360550,
                                 0.05335881596113096569914358832,
                                 0.05285886596714208397212562769,
                                 0.05236011410220643828457042296,
                                 0.05186259619060757977233889991,
                                 0.05136634740930082701566024034,
                                 0.05087140230107529017164436746,
                                 0.05037779478736472127006154389,
                                 0.04988555818071950089688354805,
                                 0.04939472519695152646764148579,
                                 0.04890532796696325145606728364,
                                 0.04841739804827163646622959881,
                                 0.04793096643623731022222820335,
                                 0.04744606357500879982214526832,
                                 0.04696271936819127349644532569,
                                 0.04648096318924884426119442319,
                                 0.04600082389164910799299968986,
                                 0.04552232981875823339173052679,
                                 0.04504550881349458293514575485,
                                 0.04457038822774852223673134909,
                                 0.04409699493157576923295544925,
                                 0.04362535532217134345067678322,
                                 0.04315549533263089840013520886,
                                 0.04268744044050595611868457304,
                                 0.04222121567615931132047129923,
                                 0.04175684563092663279966002333,
                                 0.04129435446509006104503509465,
                                 0.04083376591566938284769582591,
                                 0.04037510330403615545446895309,
                                 0.03991838954335595400584772813,
                                 0.03946364714586372609949359345,
                                 0.03901089822997705586963176427,
                                 0.03856016452725196652827347023,
                                 0.03811146738918572446164698298,
                                 0.03766482779387094932460148729,
                                 0.03722026635250518276005825998,
                                 0.03677780331575992304421834040,
                                 0.03633745858001299379558961636,
                                 0.03589925169344798158006306613,
                                 0.03546320186202434950583514230,
                                 0.03502932795532171145790167478,
                                 0.03459764851226163421443173050,
                                 0.03416818174671022207322660258,
                                 0.03374094555296463056580423865,
                                 0.03331595751112655213217607601,
                                 0.03289323489236561706570326109,
                                 0.03247279466407555742025679066,
                                 0.03205465349492588971743075950,
                                 0.03163882775981178402575314425,
                                 0.03122533354470470214191279348,
                                 0.03081418665140630602985791494,
                                 0.03040540260220805921926514737,
                                 0.02999899664445886839005560429,
                                 0.02959498375504303974130433736,
                                 0.02919337864477075483482510814,
                                 0.02879419576268320329610658557,
                                 0.02839744930027444493437578547,
                                 0.02800315319563201140132552193,
                                 0.02761132113749819734180725084,
                                 0.02722196656925393300198569492,
                                 0.02683510269282707435830466190,
                                 0.02645074247252689292588705522,
                                 0.02606889863880649541373412006,
                                 0.02568958369195485323639834430,
                                 0.02531280990572007349161086667,
                                 0.02493858933086549629820144191,
                                 0.02456693379866015828954019857,
                                 0.02419785492430511850888545365,
                                 0.02383136411029710089174513679,
                                 0.02346747254973086688686796991,
                                 0.02310619122954169250475163924,
                                 0.02274753093368928613618251687,
                                 0.02239150224628444680137999353,
                                 0.02203811555465972702323679920,
                                 0.02168738105238533021858685215,
                                 0.02133930874223143932417619130,
                                 0.02099390843907814127585533862,
                                 0.02065118977277408089915981851,
                                 0.02031116219094494770742281631,
                                 0.01997383496175287000211973872,
                                 0.01963921717660776249316960051,
                                 0.01930731775283164636986429028,
                                 0.01897814543627693432290007041,
                                 0.01865170880389964741299500544,
                                 0.01832801626628850587148480053,
                                 0.01800707607015081187407128549,
                                 0.01768889630075601902274510370,
                                 0.01737348488433786067617096776,
                                 0.01706084959045588735997060026,
                                 0.01675099803431724224088035860,
                                 0.01644393767905948303921824720,
                                 0.01613967583799523875994815969,
                                 0.01583821967681947022227207353,
                                 0.01553957621578008454038059498,
                                 0.01524375233181263543384871732,
                                 0.01495075476063982350607205568,
                                 0.01466059009883649340475357182,
                                 0.01437326480586080805215150599,
                                 0.01408878520605226388765830305,
                                 0.01380715749059719528503221030,
                                 0.01352838771946240097561517710,
                                 0.01325248182329751041211492154,
                                 0.01297944560530669353055629552,
                                 0.01270928474309030329692148328,
                                 0.01244200479045702674643363576,
                                 0.01217761117920710692453441944,
                                 0.01191610922088718520698771389,
                                 0.01165750410851730090029865035,
                                 0.01140180091829057279130324236,
                                 0.01114900461124607541531872694,
                                 0.01089912003491541123501606336,
                                 0.01065215192494346865694237481,
                                 0.01040810490668384484951567112,
                                 0.01016698349676940165582931856,
                                 0.00992879210465841250757616537,
                                 0.00969353503415674813399900010,
                                 0.00946121648491653901347893962,
                                 0.00923184055391174292697420204,
                                 0.00900541123689103663409800166,
                                 0.00878193242980844159653546765,
                                 0.00856140793023208481237378334,
                                 0.00834384143873148719164350437,
                                 0.00812923656024376349107645120,
                                 0.00791759680541910962814899291,
                                 0.00770892559194594520450001794,
                                 0.00750322624585607128061132466,
                                 0.00730050200281019585124674928,
                                 0.00710075600936417206880038413,
                                 0.00690399132421628704383432186,
                                 0.00671021091943593201331089561,
                                 0.00651941768167397780214242023,
                                 0.00633161441335517280766580614,
                                 0.00614680383385287420464015951,
                                 0.00596498858064641669566356502,
                                 0.00578617121046141691396652417,
                                 0.00561035420039330551796896388,
                                 0.00543753994901437309553388189,
                                 0.00526773077746461021640037323,
                                 0.00510092893052661632985198329,
                                 0.00493713657768484669742135455,
                                 0.00477635581416946117362089431,
                                 0.00461858866198503339771273941,
                                 0.00446383707092437383289550049,
                                 0.00431210291956771508260669470,
                                 0.00416338801626750302364169827,
                                 0.00401769410011903251929866193,
                                 0.00387502284191716180969957184,
                                 0.00373537584509933511783169098,
                                 0.00359875464667513855581452676,
                                 0.00346516071814261006362791233,
                                 0.00333459546639151985932544343,
                                 0.00320706023459383372297607800,
                                 0.00308255630308156737367701953,
                                 0.00296108489021223622749228178,
                                 0.00284264715322210094169749775,
                                 0.00272724418906740535492867814,
                                 0.00261487703525379972148652720,
                                 0.00250554667065413850895170620,
                                 0.00239925401631483847929827104,
                                 0.00229599993625097930279344240,
                                 0.00219578523823032555914369348,
                                 0.00209861067454644565965039215,
                                 0.00200447694278109997569021439,
                                 0.00191338468655606728080845586,
                                 0.00182533449627457550433249032,
                                 0.00174032690985249975195458062,
                                 0.00165836241343948757152431949,
                                 0.00157944144213016852870548453,
                                 0.00150356438066560230561083057,
                                 0.00143073156412511674449708877,
                                 0.00136094327860868452659049185,
                                 0.00129419976190998450167192831,
                                 0.00123050120418029106577255240,
                                 0.00116984774858333242084703261,
                                 0.00111223949194125604027252989,
                                 0.00105767648537183720617389835,
                                 0.00100615873491706407764229279,
                                 0.00095768620216323039167287510,
                                 0.00091225880485266458990880763,
                                 0.00086987641748722190288697407,
                                 0.00083053887192366370831123349,
                                 0.00079424595796104630983745533,
                                 0.00076099742392023915687673665,
                                 0.00073079297721569044297345717,
                                 0.00070363228491955597938545503,
                                 0.00067951497431830524060390367,
                                 0.00065844063346191651874595108,
                                 0.00064040881170577120310477292,
                                 0.00062541902024535531874497198,
                                 0.00061347073264387461300572209,
                                 0.00060456338535288767026239537,
                                 0.00059869637822605976246482575,
                                 0.00059586907502613840500388685,
                                 0.00059608080392524988356592619,
                                 0.00059933085799861434704560042,
                                 0.00060561849571177542355055769,
                                 0.00061494294140143871031435700,
                                 0.00062730338575001191322395609,
                                 0.00064269898625393786697026478,
                                 0.00066112886768591015186761355,
                                 0.00068259212255105953750071839,
                                 0.00070708781153719802590275709,
                                 0.00073461496395920583731868309,
                                 0.00076517257819764527915285660,
                                 0.00079875962213168406284377027,
                                 0.00083537503356640928357029641,
                                 0.00087501772065461195330717319,
                                 0.00091768656231312067826014310,
                                 0.00096338040863376179658470239,
                                 0.00101209808128902204100155451,
                                 0.00106383837393248856295425208,
                                 0.00111860005259413994981039288,
                                 0.00117638185607056068379955756,
                                 0.00123718249631015033043327542,
                                 0.00130100065879339760460056484,
                                 0.00136783500290828834392414493,
                                 0.00143768416232091532085530817,
                                 0.00151054674534135674694836989,
                                 0.00158642133528488926436760067,
                                 0.00166530649082860018052969887,
                                 0.00174720074636346268147205919,
                                 0.00183210261234193675766975838,
                                 0.00192001057562115759221999965,
                                 0.00201092309980177219519845086,
                                 0.00210483862556248411920297811,
                                 0.00220175557099036515928077075,
                                 0.00230167233190699202523715461,
                                 0.00240458728219046507540898214,
                                 0.00251049877409336531802075074,
                                 0.00261940513855670501890359517,
                                 0.00273130468551992640232957855,
                                 0.00284619570422700209468710491,
                                 0.00296407646352869013839573645,
                                 0.00308494521218099559553507554,
                                 0.00320880017913988996685425998,
                                 0.00333563957385233887185418003,
                                 0.00346546158654368766921831197,
                                 0.00359826438850145394374086531,
                                 0.00373404613235557504579962106,
                                 0.00387280495235515814208817873,
                                 0.00401453896464177952150688209,
                                 0.00415924626751937919756763232,
                                 0.00430692494172079615815478045,
                                 0.00445757305067098893476731930,
                                 0.00461118864074698549621688068,
                                 0.00476776974153460581594687971,
                                 0.00492731436608199981745077924,
                                 0.00508982051115004276848695185,
                                 0.00525528615745962957170378354,
                                 0.00542370926993590878669587401,
                                 0.00559508779794949661620931125,
                                 0.00576941967555471049700426888,
                                 0.00594670282172486135357409756,
                                 0.00612693514058464300032332045,
                                 0.00631011452163965661473822079,
                                 0.00649623884000310765036271373,
                                 0.00668530595661971201384247443,
                                 0.00687731371848684779474917512,
                                 0.00707225995887298831017516954,
                                 0.00727014249753345170803165364,
                                 0.00747095914092350186342833284,
                                 0.00767470768240883480130146881,
                                 0.00788138590247348438543473561,
                                 0.00809099156892518052903172295,
                                 0.00830352243709819270490150456,
                                 0.00851897625005369106396387692,
                                 0.00873735073877765700902613829,
                                 0.00895864362237637461648905833,
                                 0.00918285260826953385166932025,
                                 0.00940997539238097608364536459,
                                 0.00964000965932711197281215591,
                                 0.00987295308260304137853955961,
                                 0.01010880332476640451534302114,
                                 0.01034755803761899317367092762,
                                 0.01058921486238615041566976135,
                                 0.01083377142989398675698774597,
                                 0.01108122536074444045270436113,
                                 0.01133157426548820911871343173,
                                 0.01158481574479557953923034922,
                                 0.01184094738962518213643046324,
                                 0.01209996678139069620944909376,
                                 0.01236187149212553168697939908,
                                 0.01262665908464551278039003257,
                                 0.01289432711270958857254971460,
                                 0.01316487312117859523129212280,
                                 0.01343829464617209419558540650,
                                 0.01371458921522331034689161545,
                                 0.01399375434743219384781972628,
                                 0.01427578755361662900490090704,
                                 0.01456068633646181319205712573,
                                 0.01484844819066782855600687596,
                                 0.01513907060309542891436906532,
                                 0.01543255105291006395150406766,
                                 0.01572888701172416251608729205,
                                 0.01602807594373769652796468278,
                                 0.01633011530587704670991221769,
                                 0.01663500254793219107243512640,
                                 0.01694273511269223679662111791,
                                 0.01725331043607931588123076419,
                                 0.01756672594728086464559414453,
                                 0.01788297906888030690941412927,
                                 0.01820206721698616040418286034,
                                 0.01852398780135958570853000186,
                                 0.01884873822554039674137143546,
                                 0.01917631588697155159214879802,
                                 0.01950671817712214221667839961,
                                 0.01983994248160890128009864546,
                                 0.02017598618031624418505535537,
                                 0.02051484664751486408353274385,
                                 0.02085652125197889743456387068,
                                 0.02120100735710167743737880212,
                                 0.02154830232101009244031335401,
                                 0.02189840349667756619994902231,
                                 0.02225130823203567664242950956,
                                 0.02260701387008442955964613379,
                                 0.02296551774900120345694938558,
                                 0.02332681720224838155617400311,
                                 0.02369090955867968674800816609,
                                 0.02405779214264523508104272663,
                                 0.02442746227409532317115369549,
                                 0.02479991726868296471415130123,
                                 0.02517515443786519108682355296,
                                 0.02555317108900313082656396180,
                                 0.02593396452546088258765537132,
                                 0.02631753204670319598293902026,
                                 0.02670387094839197453298514532,
                                 0.02709297852248161476095457602,
                                 0.02748485205731319529005661872,
                                 0.02787948883770752962182459535,
                                 0.02827688614505709609730498063,
                                 0.02867704125741685836964820241,
                                 0.02907995144959398954545860331,
                                 0.02948561399323651298356829315,
                                 0.02989402615692087257360584337,
                                 0.03030518520623844515279786917,
                                 0.03071908840388100755783207012,
                                 0.03113573300972517064928748040,
                                 0.03155511628091579248906538643,
                                 0.03197723547194838269639711488,
                                 0.03240208783475050985532781313,
                                 0.03282967061876222369604418511,
                                 0.03325998107101550362399525898,
                                 0.03369301643621274502441558676,
                                 0.03412877395680429462556732611,
                                 0.03456725087306504606173951422,
                                 0.03500844442317010663674815011,
                                 0.03545235184326954615033863826,
                                 0.03589897036756223851347242914,
                                 0.03634829722836880674395256184,
                                 0.03680032965620368180117902527,
                                 0.03725506487984628558799566408,
                                 0.03771250012641134831856751795,
                                 0.03817263262141837032398323309,
                                 0.03863545958886023824178423709,
                                 0.03910097825127100541185390077,
                                 0.03956918582979284617902956035,
                                 0.04004007954424219368240212594,
                                 0.04051365661317507059251658354,
                                 0.04098991425395162214055696635,
                                 0.04146884968279986066806670862,
                                 0.04195046011487863081179550002,
                                 0.04243474276433980432585303718,
                                 0.04292169484438971343246502737,
                                 0.04341131356734983148324443897,
                                 0.04390359614471670960498869800,
                                 0.04439853978722117789756906561,
                                 0.04489614170488681964646993213,
                                 0.04539639910708772690894173062,
                                 0.04589930920260554573053046414,
                                 0.04640486919968581914791866614,
                                 0.04691307630609363603453652621,
                                 0.04742392772916859374725780697,
                                 0.04793742067587908243566327701,
                                 0.04845355235287589877981523987,
                                 0.04897231996654519682822122242,
                                 0.04949372072306078351465418198,
                                 0.05001775182843576634072219522,
                                 0.05054441048857356062052429985,
                                 0.05107369390931826359437306707,
                                 0.05160559929650440263039097585,
                                 0.05214012385600606464577941209,
                                 0.05267726479378541379369908281,
                                 0.05321701931594060437697204846,
                                 0.05375938462875309586620193980,
                                 0.05430435793873437681739400820,
                                 0.05485193645267210440272449377,
                                 0.05540211737767566618774367144,
                                 0.05595489792122117070898339536,
                                 0.05651027529119587332766279533,
                                 0.05706824669594204375793002017,
                                 0.05762880934430028159182884557,
                                 0.05819196044565228606792207638,
                                 0.05875769720996308625622471546,
                                 0.05932601684782273775878480989,
                                 0.05989691657048749195288491005,
                                 0.06047039358992044373240859711,
                                 0.06104644511883166363241117304,
                                 0.06162506837071782015233820088,
                                 0.06220626055990129802463717367,
                                 0.06279001890156881810769342711,
                                 0.06337634061180956451507893967};


double c2(double xi)
{
  if (xi<=0) {
    std::cerr << "Error in c2: xi must be positive" << std::endl;
    return 0;
  }
  double pref = 1;
  double f1,f2,f3, x2;
  unsigned i;
  // Use C2(x) = 1/xi C2(1/xi) to move argument between 0 and 1
  if (xi>1) {
    pref = 1/xi;
    xi = pref;
  }
  if (xi < 2*C2_H)
    i = 2;
  else if (xi > 1-C2_H)
    i = C2_N_POINTS - 1;
  else
    i = static_cast<int>(xi * C2_N_POINTS + double(0.5));
  //
  x2 = i*C2_H;
  f1 = C2_TABLE[i-1];
  f2 = C2_TABLE[i];
  f3 = C2_TABLE[i+1];
  //
  // singularity in xi=0 (computed analytically)
  double res = -4*std::log(xi)/3;
  //
  // interpolating finite piece
  double delta = x2-xi;
  res += f2 + 0.5*(f1-f3)*delta/C2_H + 0.5 * (f1-2*f2+f3)*delta*delta/(C2_H*C2_H);
  //
  return pref*res;
}
double Dxi_C2off(double xi)
{
  if (xi<=0) {
    std::cerr << "Error in dc2: xi must be positive" << std::endl;
    return 0;
  }
  double pref1 = 1;
  double pref2 = 1;
  double add = 0;
  double f1,f2,f3, x2;
  unsigned i;
  // Use C2'(x) = -1/xi^2( C2(1/xi) + 1/x C2'(1/x)) to move argument
  // between 0 and 1
  if (xi>1) {
    pref2 = 1/xi;
    pref1 = -pref2*pref2;
    xi = pref2;
    add = c2(xi);
  }
  if (xi < 2*C2_H)
    i = 2;
  else if (xi > 1-C2_H)
    i = C2_N_POINTS - 1;
  else
    i = static_cast<int>(xi * C2_N_POINTS + double(0.5));
  //
  x2 = i*C2_H;
  f1 = DC2_TABLE[i-1];
  f2 = DC2_TABLE[i];
  f3 = DC2_TABLE[i+1];
  //
  // singularity of function and its derivative in xi=0 (computed analytically)
  double res = - 4/(3*xi) + (8*std::log(xi))/15 - (64*xi*log(xi))/105;
  //
  // interpolating finite/smooth piece
  double delta = x2-xi;
  res += f2 + 0.5*(f1-f3)*delta/C2_H + 0.5 * (f1-2*f2+f3)*delta*delta/(C2_H*C2_H);
  //
  return pref1*(add + pref2*res) * 3./4.;
}



const unsigned CL_N_POINTS = 500;
const double CL_H = double(1)/CL_N_POINTS;

const double CL_TABLE[CL_N_POINTS+1] = {0,
                                      2.65007891088551683254121118757,
                                      2.63646805729131834024293678026,
                                      2.62399000698032337378571220537,
                                      2.61225181644556603799542084796,
                                      2.60106649888530202613550284618,
                                      2.59032371081644012276481427953,
                                      2.57995040205776692738110916556,
                                      2.56989455390008808887609569851,
                                      2.56011719771543610047705805479,
                                      2.55058802823976882589525249017,
                                      2.54128278915659775325860289757,
                                      2.53218161669409823150100585967,
                                      2.52326793864748076330599614791,
                                      2.51452771372976447531628737383,
                                      2.50594888902417881768526483423,
                                      2.49752100250465795316708422018,
                                      2.48923488512065579916495747316,
                                      2.48108243307302671794115025416,
                                      2.47305643073512242492848345812,
                                      2.46515041086503580117931604417,
                                      2.45735854277180474282569499319,
                                      2.44967554177214262171449462435,
                                      2.44209659509497577260912216119,
                                      2.43461730065635191969847344629,
                                      2.42723361602277621119642682269,
                                      2.41994181552534697378319280877,
                                      2.41273845395764862312009535152,
                                      2.40562033563878513030348948389,
                                      2.39858448788416354182453215584,
                                      2.39162813812475277392911192656,
                                      2.38474869406739866203249553156,
                                      2.37794372640632299290139340289,
                                      2.37121095368775718885115164779,
                                      2.36454822900200142530899680977,
                                      2.35795352823464444339922524153,
                                      2.35142493965463469740297824001,
                                      2.34496065465391395497909994473,
                                      2.33855895948334251684516994621,
                                      2.33221822785413534681760896111,
                                      2.32593691429412466492978297095,
                                      2.31971354816474574827676641291,
                                      2.31354672825839462979293058621,
                                      2.30743511790726790947015135218,
                                      2.30137744054439202429488193043,
                                      2.29537247566562094063500528863,
                                      2.28941905514819843686252724770,
                                      2.28351605988726199638276979724,
                                      2.27766241671658614169007978915,
                                      2.27185709558406706587316248078,
                                      2.26609910695605460993494315027,
                                      2.26038749942773789808283230221,
                                      2.25472135751946626578410345881,
                                      2.24909979964120281848682314004,
                                      2.24352197620931827675157972218,
                                      2.23798706790168289588970242695,
                                      2.23249428403854200258385847183,
                                      2.22704286107799780234090163765,
                                      2.22163206121609330742153526296,
                                      2.21626117108252609805356582589,
                                      2.21092950052392930090340650038,
                                      2.20563638146746091974720661428,
                                      2.20038116685815435707186576215,
                                      2.19516322966411448289704718075,
                                      2.18998196194420510651621014786,
                                      2.18483677397337392765973983196,
                                      2.17972709342120751644983932512,
                                      2.17465236457970810126184459872,
                                      2.16961204763664156220386009874,
                                      2.16460561799112692277118182702,
                                      2.15963256560842605396668371090,
                                      2.15469239441115195858488727613,
                                      2.14978462170434812417363513734,
                                      2.14490877763210285319520876735,
                                      2.14006440466355368122099988826,
                                      2.13525105710631016012701302032,
                                      2.13046830064548033330589788872,
                                      2.12571571190662885691033516326,
                                      2.12099287804112442052678210673,
                                      2.11629939633245221558770671537,
                                      2.11163487382217486289556222626,
                                      2.10699892695432348276765975722,
                                      2.10239118123709039795886841745,
                                      2.09781127092077712471165893126,
                                      2.09325883869102656584832090221,
                                      2.08873353537643732798762079433,
                                      2.08423501966972142960379653381,
                                      2.07976295786162487330742222795,
                                      2.07531702358688409570250289027,
                                      2.07089689758154060463751466002,
                                      2.06650226745098154708187229851,
                                      2.06213282744811586371117845447,
                                      2.05778827826113438725510888169,
                                      2.05346832681033800929358892614,
                                      2.04917268605355112612065742366,
                                      2.04490107479966820612728207785,
                                      2.04065321752990970496321603927,
                                      2.03642884422638987732311907207,
                                      2.03222769020762346506788338539,
                                      2.02804949597062093450954511465,
                                      2.02389400703924303106175186273,
                                      2.01976097381850504451953420834,
                                      2.01565015145453944908442186163,
                                      2.01156129969994260477520467483,
                                      2.00749418278424707771201776212,
                                      2.00344856928927594423873076961,
                                      1.99942423202914926871155946374,
                                      1.99542094793472586094608287475,
                                      1.99143849794227549449417086583,
                                      1.98747666688618806320164750323,
                                      1.98353524339553672784840518499,
                                      1.97961401979432200941729240580,
                                      1.97571279200523306876719670329,
                                      1.97183135945677111843577897837,
                                      1.96796952499358808169080730838,
                                      1.96412709478990128430219445868,
                                      1.96030387826585217040778938022,
                                      1.95649968800668380720664607373,
                                      1.95271433968461831349789069660,
                                      1.94894765198332134151449481287,
                                      1.94519944652484638524927676384,
                                      1.94146954779895700482301383779,
                                      1.93775778309473006696227441573,
                                      1.93406398243434782631223749723,
                                      1.93038797850899112962490744092,
                                      1.92672960661675023201333632985,
                                      1.92308870460247368739229650773,
                                      1.91946511279947952874447090065,
                                      1.91585867397305650172161432007,
                                      1.91226923326568647011058490721,
                                      1.90869663814392228577288387717,
                                      1.90514073834685841988983765860,
                                      1.90160138583613449704107923233,
                                      1.89807843474741456843698109382,
                                      1.89457174134328751449241817080,
                                      1.89108116396753638824508665587,
                                      1.88760656300072680770549298713,
                                      1.88414780081706668438091269869,
                                      1.88070474174249164376723601086,
                                      1.87727725201393245793223232287,
                                      1.87386519973972267639120319120,
                                      1.87046845486110641488866462990,
                                      1.86708688911480794767965109077,
                                      1.86372037599662635235310280381,
                                      1.86036879072601998175185480533,
                                      1.85703201021164698942717618841,
                                      1.85370991301782951736238847689,
                                      1.85040237933191047120338426028,
                                      1.84710929093247306250605449326,
                                      1.84383053115839449289810766511,
                                      1.84056598487870629470056800071,
                                      1.83731553846323492941944705949,
                                      1.83407907975399728238221083229,
                                      1.83085649803732668127418788742,
                                      1.82764768401670601089110525207,
                                      1.82445252978628539838619350155,
                                      1.82127092880506280484026539426,
                                      1.81810277587170668218171183600,
                                      1.81494796710000064127369384071,
                                      1.81180639989489082920093106051,
                                      1.80867797292911743315807087335,
                                      1.80556258612041241649846065365,
                                      1.80246014060924625098815035396,
                                      1.79937053873710703958174198833,
                                      1.79629368402529602747081974222,
                                      1.79322948115422407705347186522,
                                      1.79017783594319423606551851257,
                                      1.78713865533065605856472875668,
                                      1.78411184735491784687032457824,
                                      1.78109732113530346997445329138,
                                      1.77809498685374088134779463634,
                                      1.77510475573676990739373628873,
                                      1.77212654003795730795126598386,
                                      1.76916025302070752304637627062,
                                      1.76620580894145791634232736291,
                                      1.76326312303324770619650669807,
                                      1.76033211148965014061311224034,
                                      1.75741269144905782336722162421,
                                      1.75450478097931143581329649476,
                                      1.75160829906266242299359857590,
                                      1.74872316558106052421244069822,
                                      1.74584930130175732779473328128,
                                      1.74298662786321731782857358152,
                                      1.74013506776132815780240867334,
                                      1.73729454433590222266382963758,
                                      1.73446498175746164740239489216,
                                      1.73164630501429940722417896360,
                                      1.72883843989980918215139024774,
                                      1.72604131300007698783714079951,
                                      1.72325485168172777490541951857,
                                      1.72047898408002041156402758110,
                                      1.71771363908718466893148955048,
                                      1.71495874634099402578973171425,
                                      1.71221423621356829962958689754,
                                      1.70948003980040029418867246194,
                                      1.70675608890960083047012041004,
                                      1.70404231605135669874242494346,
                                      1.70133865442759623350955845789,
                                      1.69864503792185737214918715333,
                                      1.69596140108935321107701869276,
                                      1.69328767914723022112834310357,
                                      1.69062380796501442656508949311,
                                      1.68796972405524098992020934002,
                                      1.68532536456426277797395922928,
                                      1.68269066726323361270323069696,
                                      1.68006557053926203523190798555,
                                      1.67745001338673153080607148984,
                                      1.67484393539878327878412295387,
                                      1.67224727675895760372302985159,
                                      1.66965997823299041200567078555,
                                      1.66708198116076100323218556857,
                                      1.66451322744838774692574157418,
                                      1.66195365956046821310992910994,
                                      1.65940322051246044012532817327,
                                      1.65686185386320211478566588817,
                                      1.65432950370756452874345337093,
                                      1.65180611466923826085035441174,
                                      1.64929163189364761846357289705,
                                      1.64678600104099095116671058217,
                                      1.64428916827940402833818050546,
                                      1.64180108027824374750476968329,
                                      1.63932168420148951355097811050,
                                      1.63685092770125969970138365717,
                                      1.63438875891144066983513283755,
                                      1.63193512644142590820710159092,
                                      1.62948997936996286711454679326,
                                      1.62705326723910520553342779794,
                                      1.62462494004826815232440999139,
                                      1.62220494824838478634152899915,
                                      1.61979324273616108273064159215,
                                      1.61738977484842762994166478195,
                                      1.61499449635658597555736386724,
                                      1.61260735946114761101896063721,
                                      1.61022831678636365575977152048,
                                      1.60785732137494335019503229185,
                                      1.60549432668285951450959336834,
                                      1.60313928657423917628392731769,
                                      1.60079215531633761474968578743,
                                      1.59845288757459411291391998933,
                                      1.59612143840776775097938973829,
                                      1.59379776326315161545886303903,
                                      1.59148181797186383817413026087,
                                      1.58917355874421391798431377774,
                                      1.58687294216514281564020731692,
                                      1.58457992518973534864772132212,
                                      1.58229446513880344847861979502,
                                      1.58001651969453887692392828093,
                                      1.57774604689623403187678033694,
                                      1.57548300513606950538799855966,
                                      1.57322735315496708848920934133,
                                      1.57097905003850694805353143677,
                                      1.56873805521290773089059334434,
                                      1.56650432844106837937757301382,
                                      1.56427782981867047123691738972,
                                      1.56205851977033992360928021602,
                                      1.55984635904586692836103097307,
                                      1.55764130871648301163261165874,
                                      1.55544333017119413599941913324,
                                      1.55325238511316878830235984120,
                                      1.55106843555618002023160489695,
                                      1.54889144382110043213449249027,
                                      1.54672137253244911328641624126,
                                      1.54455818461498957403067288530,
                                      1.54240184329037772677775156444,
                                      1.54025231207385899387394593538,
                                      1.53810955477101364082038315654,
                                      1.53597353547454945326294464734,
                                      1.53384421856114089559690784184,
                                      1.53172156868831390795274337880,
                                      1.52960555079137551676612711837,
                                      1.52749613008038745210015011808,
                                      1.52539327203718298239473954647,
                                      1.52329694241242619438079230910,
                                      1.52120710722271296252738588579,
                                      1.51912373274771286860216057046,
                                      1.51704678552735134772965036076,
                                      1.51497623235903135274167060287,
                                      1.51291204029489384363916573846,
                                      1.51085417663911642363713240185,
                                      1.50880260894524945755396265350,
                                      1.50675730501358902224406150004,
                                      1.50471823288858605236781744884,
                                      1.50268536085629105805556485740,
                                      1.50065865744183380396138831010,
                                      1.49863809140693735182750477312,
                                      1.49662363174746588099925822454,
                                      1.49461524769100571335293987412,
                                      1.49261290869447898083190717109,
                                      1.49061658444178938523876401839,
                                      1.48862624484149951111038449389,
                                      1.48664186002453916341577695435,
                                      1.48466340034194421247142932676,
                                      1.48269083636262543887186279279,
                                      1.48072413887116688139144904894,
                                      1.47876327886565320073370821620,
                                      1.47680822755552558169269283283,
                                      1.47485895635946570575387768215,
                                      1.47291543690330733540522838183,
                                      1.47097764101797506045864623386,
                                      1.46904554073744976550344099459,
                                      1.46711910829676038623235659610,
                                      1.46519831613000153080229406721,
                                      1.46328313686837655062140998146,
                                      1.46137354333826565299673427493,
                                      1.45946950855931865593671735818,
                                      1.45757100574257199308590958410,
                                      1.45567800828858958427888482041,
                                      1.45379048978562719454199908295,
                                      1.45190842400781991154895067301,
                                      1.45003178491339237855358074149,
                                      1.44816054664289142668500235250,
                                      1.44629468351744075719993445425,
                                      1.44443417003701733084889377687,
                                      1.44257898087874912793040183478,
                                      1.44072909089523394888422884499,
                                      1.43888447511287893141445143040,
                                      1.43704510873026046613917666536,
                                      1.43521096711650419863951493722,
                                      1.43338202580968481152900825075,
                                      1.43155826051524528578938833023,
                                      1.42973964710443534612231166350,
                                      1.42792616161276880045257283815,
                                      1.42611778023849948898912699817,
                                      1.42431447934111556340887089233,
                                      1.42251623543985182177727618065,
                                      1.42072302521221982976229966349,
                                      1.41893482549255556353610130583,
                                      1.41715161327058431449550012522,
                                      1.41537336569000260056923948530,
                                      1.41360006004707683342039995803,
                                      1.41183167378925849529800814008,
                                      1.41006818451381558364529960074,
                                      1.40830956996648008583539885774,
                                      1.40655580804011125058051547572,
                                      1.40480687677337442665020255991,
                                      1.40306275434943524353980621299,
                                      1.40132341909466891265392235127,
                                      1.39958884947738443141338891248,
                                      1.39785902410656347645994460096,
                                      1.39613392173061377582199845286,
                                      1.39441352123613675351974852224,
                                      1.39269780164670924362988744925,
                                      1.39098674212167907430101718104,
                                      1.38928032195497432561230063026,
                                      1.38757852057392606850139814155,
                                      1.38588131753810439525492368697,
                                      1.38418869253816755525702213641,
                                      1.38250062539472401283068831091,
                                      1.38081709605720724708355665542,
                                      1.37913808460276311668648643118,
                                      1.37746357123514961547071487358,
                                      1.37579353628364884762897872404,
                                      1.37412796020199105414910822176,
                                      1.37246682356729052489643964162,
                                      1.37081010707899323349520362153,
                                      1.36915779155783603484002678693,
                                      1.36750985794481726769800347725,
                                      1.36586628730017860744059344753,
                                      1.36422706080239801647399261634,
                                      1.36259215974719364241769202363,
                                      1.36096156554653851651474307885,
                                      1.35933525972768590714481675771,
                                      1.35771322393220518565348709804,
                                      1.35609543991502806400926692206,
                                      1.35448188954350506605473392755,
                                      1.35287255479647209633054254101,
                                      1.35126741776332697262213287793,
                                      1.34966646064311579050941236618,
                                      1.34806966574362899029046609662,
                                      1.34647701548050699870229587229,
                                      1.34488849237635531987552195955,
                                      1.34330407905986895193671259471,
                                      1.34172375826496600761232196040,
                                      1.34014751282993041909288641851,
                                      1.33857532569656360928590178282,
                                      1.33700717990934501342141403923,
                                      1.33544305861460133677651754103,
                                      1.33388294505968443605436680761,
                                      1.33232682259215771369065268455,
                                      1.33077467465899091606643682053,
                                      1.32922648480576322828143063200,
                                      1.32768223667587455978688142228,
                                      1.32614191400976491679280957458,
                                      1.32460550064414175895103281196,
                                      1.32307298051121523937380844382,
                                      1.32154433763794122857860065425,
                                      1.32001955614527202445300226574,
                                      1.31849862024741465181076108498,
                                      1.31698151425109665656071930644,
                                      1.31546822255483930093579758977,
                                      1.31395872964823806762945839710,
                                      1.31245302011125038206286932693,
                                      1.31095107861349046335774844809,
                                      1.30945288991353121591809083071,
                                      1.30795843885821307482911855478,
                                      1.30646771038195971956432484140,
                                      1.30498068950610057175184567683,
                                      1.30349736133819999399002841450,
                                      1.30201771107139310791940557025,
                                      1.30054172398372815095474304073,
                                      1.29906938543751529225682561913,
                                      1.29760068087868182967957021156,
                                      1.29613559583613369056431095850,
                                      1.29467411592112316037006427641,
                                      1.29321622682662276422663095562,
                                      1.29176191432670522757689395059,
                                      1.29031116427592944313598341695,
                                      1.28886396260873237243845609019,
                                      1.28742029533882681127061782284,
                                      1.28598014855860494929394210022,
                                      1.28454350843854765515753245917,
                                      1.28311036122663941937306465620,
                                      1.28168069324778888818493995362,
                                      1.28025449090325492261179203016,
                                      1.27883174067007811776331818617,
                                      1.27741242910051771844894566623,
                                      1.27599654282149386799238473061,
                                      1.27458406853403512804894409609,
                                      1.27317499301273120809086805924,
                                      1.27176930310519084408016867764,
                                      1.27036698573150476668873576369,
                                      1.26896802788371370025217150254,
                                      1.26757241662528133445706914736,
                                      1.26618013909057221156158505298,
                                      1.26479118248433447273638366019,
                                      1.26340553408118740788760423569,
                                      1.26202318122511375408563954030,
                                      1.26064411132895668847345862667,
                                      1.25926831187392146226617240733,
                                      1.25789577040908162317975060712,
                                      1.25652647455088977434146682384,
                                      1.25516041198269281843798485276,
                                      1.25379757045425163654921005563,
                                      1.25243793778126515179731602568,
                                      1.25108150184489872861091665008,
                                      1.24972825059131685906438040563,
                                      1.24837817203122008840196691234,
                                      1.24703125423938613249599113511,
                                      1.24568748535421514061777013032,
                                      1.24434685357727905751985916907,
                                      1.24300934717287503943821312730,
                                      1.24167495446758287922358639952,
                                      1.24034366384982639640287801604,
                                      1.23901546376943874855340251776,
                                      1.23769034273723162094638257746,
                                      1.23636828932456825198047425558,
                                      1.23504929216294025248200490377,
                                      1.23373333994354817749597877764,
                                      1.23242042141688580973093508409,
                                      1.23111052539232811435157322914,
                                      1.22980364073772282533583333605,
                                      1.22849975637898562412797675492,
                                      1.22719886129969887182628861595,
                                      1.22590094454071385664345714098,
                                      1.22460599519975651886960444108,
                                      1.22331400243103661605248034307,
                                      1.22202495544486029158661134100,
                                      1.22073884350724601037334553327,
                                      1.21945565593954382567687345150,
                                      1.21817538211805794175755372345,
                                      1.21689801147367253731334894697,
                                      1.21562353349148081520299613527,
                                      1.21435193771041724436081056954,
                                      1.21308321372289296024286264568,
                                      1.21181735117443429056778299656,
                                      1.21055433976332437353274840842,
                                      1.20929416924024783609638440339,
                                      1.20803682940793850032549242036,
                                      1.20678231012083008620177094293,
                                      1.20553060128470987967814944562,
                                      1.20428169285637533516208854728,
                                      1.20303557484329358198531434196,
                                      1.20179223730326380479604280759,
                                      1.20055167034408246818090301051,
                                      1.19931386412321135618957535993,
                                      1.19807880884744839779571157526,
                                      1.19684649477260124968308282924,
                                      1.19561691220316360809619663023,
                                      1.19439005149199422183991474918,
                                      1.19316590303999857885297567781,
                                      1.19194445729581323911585601772,
                                      1.19072570475549278698417465794,
                                      1.18950963596219937636492896932,
                                      1.18829624150589484247432947679,
                                      1.18708551202303535423294312757,
                                      1.18587743819626858166633855473,
                                      1.18467201075413335298752151103,
                                      1.18346922047076177634122547976,
                                      1.18226905816558380148965064155,
                                      1.18107151470303419701459191971,
                                      1.17987658099226191890213054343,
                                      1.17868424798684184666324905708,
                                      1.17749450668448886342693138513,
                                      1.17630734812677425672159069868,
                                      1.17512276339884441693609055364,
                                      1.17394074362914181072325010229,
                                      1.17276127998912820687661205337,
                                      1.17158436369301013247546133696,
                                      1.17040998599746653735367094130,
                                      1.16923813820137864520497592578,
                                      1.16806881164556196989079297014};

double DCL_TABLE[CL_N_POINTS+1] = {0,
                                 1.00102231697141305826272447400,
                                 0.992694846460300392903116707974,
                                 0.986512178513154347990071469589,
                                 0.981325095158862747140946471729,
                                 0.976734564086407815468098913134,
                                 0.972549715999340259994314937916,
                                 0.968662828661346242497153302753,
                                 0.965006483163473110770274678626,
                                 0.961535364020751265913789302154,
                                 0.958217373544217620198954254643,
                                 0.955028843472931905293546412284,
                                 0.951951755990867684696970783375,
                                 0.948972034751117025594478050399,
                                 0.946078443800719908658131515138,
                                 0.943261850447928286196444375381,
                                 0.940514715666610673029950718136,
                                 0.937830732041859341681610398209,
                                 0.935204560412423413508702419292,
                                 0.932631634337831877961482893520,
                                 0.930108012284005365496000591957,
                                 0.927630264084582006715868378104,
                                 0.925195382479501646922622886334,
                                 0.922800713305424393644775145345,
                                 0.920443899765859054980891543533,
                                 0.918122837472954786288899512853,
                                 0.915835637831084994776252189560,
                                 0.913580597952694483048280780443,
                                 0.911356175741803149026381960006,
                                 0.909160969104141021629654542839,
                                 0.906993698481253381669612479580,
                                 0.904853192083588671834571058513,
                                 0.902738373331472608430562592313,
                                 0.900648250114796633074721441872,
                                 0.898581905560574808390190210197,
                                 0.896538490058246456175021031926,
                                 0.894517214340067382483727860291,
                                 0.892517343451322336814312960512,
                                 0.890538191474758448936844148157,
                                 0.888579116897339942986856340503,
                                 0.886639518526480532503024010042,
                                 0.884718831878325003793211926158,
                                 0.882816525973193092865222137327,
                                 0.880932100483558162535309236291,
                                 0.879065083188369110962112203835,
                                 0.877215027694494585122150053081,
                                 0.875381511391855372776556770915,
                                 0.873564133613636246386093474241,
                                 0.871762513977009334969455098713,
                                 0.869976290883198587473515900390,
                                 0.868205120158582540861546786014,
                                 0.866448673820962008151482379991,
                                 0.864706638957184727255996040336,
                                 0.862978716700080957706961474305,
                                 0.861264621294171968217517492781,
                                 0.859564079240907974356108205197,
                                 0.857876828515306885407125976602,
                                 0.856202617846827952818230144576,
                                 0.854541206058148134593468007280,
                                 0.852892361456232922373040739830,
                                 0.851255861270723600785479713167,
                                 0.849631491135212917615172298148,
                                 0.848019044607462300617649572296,
                                 0.846418322725035657493558275591,
                                 0.844829133593195566060869708718,
                                 0.843251292002234204153658328726,
                                 0.841684619071699561472710383357,
                                 0.840128941919232338020600086989,
                                 0.838584093351954763047704223471,
                                 0.837049911578553052524334893538,
                                 0.835526239940373534657619452602,
                                 0.834012926660011347567162030585,
                                 0.832509824606012415152414729224,
                                 0.831016791072436184117499759797,
                                 0.829533687572140134362250754931,
                                 0.828060379642748902289438893374,
                                 0.826596736664362329145967841302,
                                 0.825142631688139040595475545838,
                                 0.823697941274966308363376038083,
                                 0.822262545343493841907737110743,
                                 0.820836327026869599533679951557,
                                 0.819419172537570390679960343095,
                                 0.818010971039769577900365423424,
                                 0.816611614528729119868398545286,
                                 0.815220997716744004215837989523,
                                 0.813839017925204224936098912076,
                                 0.812465574982373239172940946784,
                                 0.811100571126512626115459355805,
                                 0.809743910914010763129876414630,
                                 0.808395501132198995398350377658,
                                 0.807055250716562240829907071417,
                                 0.805723070672072452267437234986,
                                 0.804398873998393042177249072615,
                                 0.803082575618720429519093512543,
                                 0.801774092312045445357709230506,
                                 0.800473342648632568525601761769,
                                 0.799180246928528977077443713106,
                                 0.797894727122928304969954205327,
                                 0.796616706818225885061631669658,
                                 0.795346111162613228162085161352,
                                 0.794082866815069613823704659751,
                                 0.792826901896618024476972125124,
                                 0.791578145943721306062153234375,
                                 0.790336529863702445002421279437,
                                 0.789101985892080267135165473913,
                                 0.787874447551718738016613476982,
                                 0.786653849613694420301299064326,
                                 0.785440128059792563097565563580,
                                 0.784233220046547797103258062464,
                                 0.783033063870750521444611169236,
                                 0.781839598936344824030774088249,
                                 0.780652765722648204778985612022,
                                 0.779472505753827495727355431028,
                                 0.778298761569569217112694606781,
                                 0.777131476696886195247622696489,
                                 0.775970595623005615985275067174,
                                 0.774816063769286814581607692159,
                                 0.773667827466120025243420811752,
                                 0.772525833928760046626784319189,
                                 0.771390031234051336841900416809,
                                 0.770260368298003445826707348801,
                                 0.769136794854177935955409385361,
                                 0.768019261432850044204191549446,
                                 0.766907719340910311011899762892,
                                 0.765802120642473251281532609537,
                                 0.764702418140161880194231905484,
                                 0.763608565357038538428579430010,
                                 0.762520516519153995178767536239,
                                 0.761438226538688249686419684795,
                                 0.760361650997657808984833708341,
                                 0.759290746132165496886102516443,
                                 0.758225468817170052185742522250,
                                 0.757165776551753907494891020913,
                                 0.756111627444868608560579061930,
                                 0.755062980201538341597133749207,
                                 0.754019794109502986922704220372,
                                 0.752982029026283014693379941542,
                                 0.751949645366649386116996540386,
                                 0.750922604090482424337390754289,
                                 0.749900866691004376117855172552,
                                 0.748884395183371101229144587548,
                                 0.747873152093609003586126945832,
                                 0.746867100447883959029797190382,
                                 0.745866203762089601410675742474,
                                 0.744870426031742903341230289597,
                                 0.743879731722175532558206657977,
                                 0.742894085759009981053601811822,
                                 0.741913453518909953661261114758,
                                 0.740937800820594967181819623853,
                                 0.739967093916109551848095074693,
                                 0.739001299482337865338497874995,
                                 0.738040384612754926912854904038,
                                 0.737084316809406056767554917704,
                                 0.736133063975106464504071056048,
                                 0.735186594405853271725530042182,
                                 0.734244876783442578203607802062,
                                 0.733307880168284489715332870771,
                                 0.732375573992409319402366813865,
                                 0.731447928052658454167126855162,
                                 0.730524912504053643954515601907,
                                 0.729606497853338725492783298590,
                                 0.728692654952688033856937190774,
                                 0.727783354993575985707738153273,
                                 0.726878569500802537845761266929,
                                 0.725978270326669434365225430044,
                                 0.725082429645302355725471686181,
                                 0.724191019947114273977526057067,
                                 0.723304014033405500658796865027,
                                 0.722421385011096087943370715884,
                                 0.721543106287586409926090808400,
                                 0.720669151565741909819476288454,
                                 0.719799494838998150725240742020,
                                 0.718934110386582452857552523678,
                                 0.718072972768848538974583463321,
                                 0.717216056822720742631317801318,
                                 0.716363337657244460995825813756,
                                 0.715514790649239655652857467296,
                                 0.714670391439054321317089535203,
                                 0.713830115926414953943780967621,
                                 0.712993940266371156593628995937,
                                 0.712161840865331623805297355361,
                                 0.711333794377188843365528493340,
                                 0.710509777699529948443865521949,
                                 0.709689767969931243267151922478,
                                 0.708873742562334012028556071777,
                                 0.708061679083499303727917401250,
                                 0.707253555369539465286926787118,
                                 0.706449349482524271727895141039,
                                 0.705649039707159575594626343259,
                                 0.704852604547536468266766985075,
                                 0.704060022723949013506522654121,
                                 0.703271273169778678603758918905,
                                 0.702486335028443650970944704894,
                                 0.701705187650411288095940314586,
                                 0.700927810590272006495493339062,
                                 0.700154183603872970827416271322,
                                 0.699384286645509997711720762894,
                                 0.698618099865176140172693266761,
                                 0.697855603605865468032763884974,
                                 0.697096778400930607148562268558,
                                 0.696341604971492646159282845911,
                                 0.695590064223902063493113370090,
                                 0.694842137247249369821137581401,
                                 0.694097805310924202028519447037,
                                 0.693357049862221644155398007958,
                                 0.692619852523994588707190903903,
                                 0.691886195092350988305439056943,
                                 0.691156059534394882902686929464,
                                 0.690429427986010121772329253510,
                                 0.689706282749685732258530813981,
                                 0.688986606292381918881548478505,
                                 0.688270381243435706887123491331,
                                 0.687557590392505273750009953273,
                                 0.686848216687552040534084728699,
                                 0.686142243232859622415846693574,
                                 0.685439653287088764133635533984,
                                 0.684740430261367411669022442611,
                                 0.684044557717415096135336036194,
                                 0.683352019365700829675404698923,
                                 0.682662799063633736189044525398,
                                 0.681976880813785661951900404613,
                                 0.681294248762145032680902231239,
                                 0.680614887196401244376484456800,
                                 0.679938780544258895355262225028,
                                 0.679265913371781186305317628956,
                                 0.678596270381761833974766183132,
                                 0.677929836412124862266924248501,
                                 0.677266596434351652085247134552,
                                 0.676606535551934648270352694618,
                                 0.675949638998857138421062383536,
                                 0.675295892138098534311777550089,
                                 0.674645280460164602029120708339,
                                 0.673997789581642101870266404705,
                                 0.673353405243777313491656394388,
                                 0.672712113311077935787001645734,
                                 0.672073899769937864524084714007,
                                 0.671438750727284363896690048750,
                                 0.670806652409247160866170593277,
                                 0.670177591159849003491262630396,
                                 0.669551553439717236388762408288,
                                 0.668928525824815958044997576382,
                                 0.668308495005198335921552916939,
                                 0.667691447783778666180824624922,
                                 0.667077371075123775409576212707,
                                 0.666466251904263371953183176607,
                                 0.665858077405518964400669513465,
                                 0.665252834821350974391518244433,
                                 0.664650511501223680259734289270,
                                 0.664051094900487637098515460497,
                                 0.663454572579279227629536799368,
                                 0.662860932201437006803308181080,
                                 0.662270161533434511350015786462,
                                 0.661682248443329213552067222925,
                                 0.661097180899727306328275866070,
                                 0.660514946970764014312989080406,
                                 0.659935534823099132988945144109,
                                 0.659358932720927505097415727815,
                                 0.658785129025004150510169814958,
                                 0.658214112191683772511641393893,
                                 0.657645870771974370012813111686,
                                 0.657080393410604691606922902485,
                                 0.656517668845105273587116948492,
                                 0.655957685904902810083350826915,
                                 0.655400433510427609345714698892,
                                 0.654845900672233895909262271050,
                                 0.654294076490132723926500297537,
                                 0.653744950152337272352905378696,
                                 0.653198510934620297922961283925,
                                 0.652654748199483526963867187317,
                                 0.652113651395338772065706570344,
                                 0.651575210055700564464783288864,
                                 0.651039413798390097705170330257,
                                 0.650506252324750282726278564357,
                                 0.649975715418871718985298910786,
                                 0.649447792946829390566429863484,
                                 0.648922474855929900457472867537,
                                 0.648399751173969060292136749273,
                                 0.647879612008499656866596545660,
                                 0.647362047546109220644744622733,
                                 0.646847048051707625271285858884,
                                 0.646334603867824350818390960107,
                                 0.645824705413915247102957816897,
                                 0.645317343185678636930467238341,
                                 0.644812507754380602550688971978,
                                 0.644310189766189301952738233086,
                                 0.643810379941518164884756006077,
                                 0.643313069074377821659257733631,
                                 0.642818248031736620901352965612,
                                 0.642325907752889595415893055619,
                                 0.641836039248835738293389539716,
                                 0.641348633601663454245424131384,
                                 0.640863681963944053960333918172,
                                 0.640381175558133162001226348269,
                                 0.639901105675979911432816771763,
                                 0.639423463677943800963082626239,
                                 0.638948240992619092922128192592,
                                 0.638475429116166632875729096476,
                                 0.638005019611752974086496900505,
                                 0.637537004108996692393137320517,
                                 0.637071374303421779379484405208,
                                 0.636608121955918003951440378978,
                                 0.636147238892208134633150802667,
                                 0.635688717002321917035164097394,
                                 0.635232548240076703038384579779,
                                 0.634778724622564630279706256165,
                                 0.634327238229646252519645522537,
                                 0.633878081203450523420368393831,
                                 0.633431245747881038165486095104,
                                 0.632986724128128439212087665256,
                                 0.632544508670188894281868551352,
                                 0.632104591760388556473043179344,
                                 0.631666965844913918109105821478,
                                 0.631231623429347971635503022487,
                                 0.630798557078212092531945426676,
                                 0.630367759414513560827428923026,
                                 0.629939223119298639388036314236,
                                 0.629512940931211128695203796996,
                                 0.629088905646056319345285816097,
                                 0.628667110116370264980834474747,
                                 0.628247547250994299810896441700,
                                 0.627830210014654726292666491174,
                                 0.627415091427547599930843078320,
                                 0.627002184564928539504804462008,
                                 0.626591482556707492358037517752,
                                 0.626182978587048385679856870572,
                                 0.625776665893973595977079027240,
                                 0.625372537768973170173673572244,
                                 0.624970587556618732990189719824,
                                 0.624570808654182016442620473630,
                                 0.624173194511257948462968227634,
                                 0.623777738629392238781746343029,
                                 0.623384434561713401326604737200,
                                 0.622993275912569153481800248759,
                                 0.622604256337167133620924209650,
                                 0.622217369541219879370713782317,
                                 0.621832609280594010087458040599,
                                 0.621449969360963558029997121053,
                                 0.621069443637467393695120951021,
                                 0.620691026014370691742806700013,
                                 0.620314710444730384880680995539,
                                 0.619940490930064553999830475081,
                                 0.619568361520025703758075793205,
                                 0.619198316312077873692520542118,
                                 0.618830349451177535811026228626,
                                 0.618464455129458230462674181755,
                                 0.618100627585918893120670250827,
                                 0.617738861106115825527932452922,
                                 0.617379150021858265456168587880,
                                 0.617021488710907510113983014864,
                                 0.616665871596679549008821856939,
                                 0.616312293147951162821736575234,
                                 0.615960747878569445593370256761,
                                 0.615611230347164708244592916875,
                                 0.615263735156866722166166431999,
                                 0.614918256955024262309032435174,
                                 0.614574790432927909890605169185,
                                 0.614233330325536075503125188849,
                                 0.613893871411204204067990211838,
                                 0.613556408511417123725319832445,
                                 0.613220936490524501381117176062,
                                 0.612887450255479368255541488356,
                                 0.612555944755579679385272601163,
                                 0.612226414982212871630995754919,
                                 0.611898855968603385327921220260,
                                 0.611573262789563115293228853516,
                                 0.611249630561244757469638101408,
                                 0.610927954440898018039187833698,
                                 0.610608229626628652386000532577,
                                 0.610290451357160301821528786637,
                                 0.609974614911599096510760050109,
                                 0.609660715609200993553304060569,
                                 0.609348748809141819679416641422,
                                 0.609038709910289988518029134222,
                                 0.608730594350981862881954644125,
                                 0.608424397608799732994825962106,
                                 0.608120115200352382055175997565,
                                 0.607817742681058210995585718586,
                                 0.607517275644930894749178342861,
                                 0.607218709724367542782108846802,
                                 0.606922040589939337089257488816,
                                 0.606627263950184621281253545380,
                                 0.606334375551404414814395363888,
                                 0.606043371177460326831155740521,
                                 0.605754246649574844487924305479,
                                 0.605466997826133971048594090139,
                                 0.605181620602492189417697187497,
                                 0.604898110910779727175180300218,
                                 0.604616464719712099556727471056,
                                 0.604336678034401907198923540930,
                                 0.604058746896172865837643763507,
                                 0.603782667382376045510985241447,
                                 0.603508435606208297174954078674,
                                 0.603236047716532844991115007492,
                                 0.602965499897702022890621472457,
                                 0.602696788369382134358594615855,
                                 0.602429909386380414716827426433,
                                 0.602164859238474075511370902307,
                                 0.601901634250241410934825235089,
                                 0.601640230780894946531220983022,
                                 0.601380645224116610744340714568,
                                 0.601122874007894910178305993180,
                                 0.600866913594364089742340799384,
                                 0.600612760479645259149921206475,
                                 0.600360411193689467536130757617,
                                 0.600109862300122708246057766591,
                                 0.599861110396092836131588784885,
                                 0.599614152112118379974063772819,
                                 0.599368984111939232926053089137,
                                 0.599125603092369204137082309235,
                                 0.598884005783150414995554213957,
                                 0.598644188946809523682482304696,
                                 0.598406149378515761992039318068,
                                 0.598169883905940768629418078488,
                                 0.597935389389120203448179550004,
                                 0.597702662720317127337201350493,
                                 0.597471700823887132711614844989,
                                 0.597242500656145209802804207661,
                                 0.597015059205234334179708934699,
                                 0.596789373490995761167393076861,
                                 0.596565440564841013059189325457,
                                 0.596343257509625545245761961491,
                                 0.596122821439524077608226080703,
                                 0.595904129499907577743076572401,
                                 0.595687178867221882804182844914,
                                 0.595471966748867946961556729007,
                                 0.595258490383083701688062544772,
                                 0.595046747038827516293769929019,
                                 0.594836734015663246333310411279,
                                 0.594628448643646857714445429408,
                                 0.594421888283214614536142862044,
                                 0.594217050325072818881846463871,
                                 0.594013932190089090988361954832,
                                 0.593812531329185178402927992264,
                                 0.593612845223231282930641849414,
                                 0.593414871382941894361519312673,
                                 0.593218607348773120151136069187,
                                 0.593024050690821500411072684907,
                                 0.592831199008724297745315215505,
                                 0.592640049931561251646395662849,
                                 0.592450601117757787340437093499,
                                 0.592262850254989669143442591923,
                                 0.592076795060089088562179783541,
                                 0.591892433278952177541907040357,
                                 0.591709762686447937430006459831,
                                 0.591528781086328574389374265978,
                                 0.591349486311141232158212615551,
                                 0.591171876222141113213708330060,
                                 0.590995948709205979556013496703,
                                 0.590821701690752024485999137906,
                                 0.590649133113651106905474477360,
                                 0.590478240953149339821988271340,
                                 0.590309023212787024891992089667,
                                 0.590141477924319924986084519836,
                                 0.589975603147641866908305582139,
                                 0.589811396970708666548047102826,
                                 0.589648857509463368888121699406,
                                 0.589487982907762795435924088036,
                                 0.589328771337305391786456740618,
                                 0.589171220997560368166310038326,
                                 0.589015330115698125946516968068,
                                 0.588861096946521963249575521746,
                                 0.588708519772401052911879184273,
                                 0.588557596903204686197347613396,
                                 0.588408326676237775791235692548,
                                 0.588260707456177611734948950843,
                                 0.588114737635011864093235781476,
                                 0.587970415631977826274390367939,
                                 0.587827739893502893052112702008,
                                 0.587686708893146267464461053747,
                                 0.587547321131541890890924797544,
                                 0.587409575136342590733068246759,
                                 0.587273469462165440247475321500,
                                 0.587139002690538325201886280085,
                                 0.587006173429847712146486797120,
                                 0.586874980315287613212311393433,
                                 0.586745422008809742467682260196,
                                 0.586617497199074858981545146044,
                                 0.586491204601405291859510106203,
                                 0.586366542957738642634380108885,
                                 0.586243511036582660507977975711,
                                 0.586122107632971286055184784626,
                                 0.586002331568421859114303243959,
                                 0.585884181690893486700179895737,
                                 0.585767656874746566887982255399,
                                 0.585652756020703464726152772104,
                                 0.585539478055810336346872132644,
                                 0.585427821933400097551380979523,
                                 0.585317786633056533255752337148,
                                 0.585209371160579544290197434373,
                                 0.585102574547951528151745406320,
                                 0.584997395853304890416182524792,
                                 0.584893834160890683620488864465,
                                 0.584791888581048370531689136251,
                                 0.584691558250176708822059046392,
                                 0.584592842330705754274017974653,
                                 0.584495740011069979740811786708,
                                 0.584400250505682507191264764057,
                                 0.584306373054910450268475296244,
                                 0.584214106925051364893364273786,
                                 0.584123451408310805544475983718,
                                 0.584034405822780984945396485068};


double cl(double xi)
{
  if(xi==0) return 0;
  if (xi<0) {
    std::cerr << "Error in cl: xi must be positive; was xi = " << xi << std::endl;
    return 0;
  }

  double pref = 1;
  double f1,f2,f3, x2;
  unsigned i;

  // Use CL(x) = 1/xi CL(1/xi) to move argument between 0 and 1
  if (xi>1) {
    pref = 1/xi;
    xi = pref;
  }

  if (xi < 2*CL_H)
    i = 2;
  else if (xi > 1-CL_H)
    i = CL_N_POINTS - 1;
  else
    i = static_cast<int>(xi * CL_N_POINTS + double(0.5));

  x2 = i*CL_H;
  f1 = CL_TABLE[i-1];
  f2 = CL_TABLE[i];
  f3 = CL_TABLE[i+1];

  // interpolating
  double delta = x2-xi;
  double res = f2 + 0.5*(f1-f3)*delta/CL_H + 0.5 * (f1-2*f2+f3)*delta*delta/(CL_H*CL_H);

  return pref*res;
}


double Dxi_CLoff(double xi)
{
  if(xi==0) return 0;
  if (xi<0) {
    std::cerr << "Error in dcl: xi must be positive; was xi = " << xi << std::endl;
    return 0;
  }

  double pref1 = 1;
  double pref2 = 1;
  double add = 0;
  double f1,f2,f3, x2;
  unsigned i;

  // Use CL'(x) = -1/xi^2( CL(1/xi) + 1/x CL'(1/x)) to move argument
  // between 0 and 1
  if (xi>1) {
    pref2 = 1/xi;
    pref1 = -pref2*pref2;
    xi = pref2;
    add = cl(xi);
  }

  if (xi < 2*CL_H)
    i = 2;
  else if (xi > 1-CL_H)
    i = CL_N_POINTS - 1;
  else
    i = static_cast<int>(xi * CL_N_POINTS + double(0.5));

  x2 = i*CL_H;
  f1 = DCL_TABLE[i-1];
  f2 = DCL_TABLE[i];
  f3 = DCL_TABLE[i+1];

  // interpolating piece
  double delta = x2-xi;
  double res = f2 + 0.5*(f1-f3)*delta/CL_H + 0.5 * (f1-2*f2+f3)*delta*delta/(CL_H*CL_H);
  res *= (std::log(xi)-1); // we factored out a log

  return pref1*(add + pref2*res)  *3./8.;
}



};

