/* -----------------------------------------

   _  _ ____ _    _    
   |__| |___ |    |    
   |  | |___ |___ |___ 
   
   HELL: High-Energy Large Logarithms

   Author: Marco Bonvini

   Computes Delta P_res = P_res - P_FixedOrder
   from a file containing parameters

   ----------------------------------------- */

#include <iomanip>
#include <iostream>
#include <fstream>
#include <string>
#include <sstream>
#include <cmath>
#include <cstdlib>

#include "hell.hh"
#include "Pqg.hh"
#include "CoefficientFunctions.hh"
#include "integration.hh"


using namespace std;



namespace HELL {

  dcomplex DeltaGamma(dcomplex Delta_gamma, dcomplex Delta_gammadot, dcomplex N, double as, double nf, dcomplex Delta_gammaNLO) {
    return Delta_gamma;
  }


  // **************************
  // ***   fixed nf part    ***
  // **************************


  HELLnf::~HELLnf() {
  }


  void HELLnf::Init(string datapath) {
    datapath_ = datapath;
    //string pyexec = "python "+datapath+"/gen_info.py "+datapath;
    //system(pyexec.c_str());
    if(_order==0) {
      ostringstream filename;
      filename << datapath << "/lo_nf" << _nf << ".info";
      ifstream info(filename.str().c_str());
      double astmp;
      while(info.good()) {
	info >> astmp;
	_alphas.push_back(astmp);
      }
      _alphas.pop_back();
      _alphas2.resize(_alphas.size());
      _alphas2 = _alphas;
    } else
      if (_order == 1) {
      ostringstream filename;
      filename << datapath << "/nlo_nf" << _nf << ".info";
      ifstream info(filename.str().c_str());
      filename.str("");  filename.clear();
      filename << datapath << "/lobatnlo_nf" << _nf << ".info";
      ifstream info2(filename.str().c_str());
      double astmp;
      while(info.good()) {
	info >> astmp;
	_alphas.push_back(astmp);
	info2 >> astmp;
	_alphas2.push_back(astmp);
      }
      _alphas.pop_back();
      _alphas2.pop_back();
    }
  }

  void HELLnf::GetAvailableAlphas(vector<double> &as1, vector<double> &as2) {
    as1.resize(_alphas.size());
    as2.resize(_alphas2.size());
    as1 = _alphas;
    as2 = _alphas2;
    return;
  }


  int HELLnf::alphas_interpolation(double as, vector<double> vas, double &factor) {
    int k = 0;
    if(as < vas[0] || as > vas[vas.size()-1]) {
      cout << "ERROR: alpha_s=" << as << " out of interpolation range [" << vas[0] << ", " << vas[vas.size()-1] << "]" << endl;
      exit(22);
    }
    for(int i=1; i<vas.size(); i++) {
      if(as <= vas[i]) break;
      k++;
    }
    //cout << alphas[k] << " < " << as << " < " << alphas[k+1] << endl;
    factor = (as-vas[k]) / (vas[k+1]-vas[k]);
    return k;
  }

  string sas(double as) {
    ostringstream os;
    if     (as<0.01) os << "000" << int(1000*as);
    else if(as<0.1 ) os << "00"  << int(1000*as);
    else if(as<1.  ) os << "0"   << int(1000*as);
    else os << int(1000*as);
    return os.str();
  }

  void HELLnf::ReadTable(int k) {
    ostringstream filename;
    string sord = (_order == 1 ? "nlo" : "lo");
    vector<int> kk;
    for(int j=0; j<2; j++) {
      itHTP = HTP.find(k+j);
      if(itHTP == HTP.end()) kk.push_back(k+j);
    }
    for(int i=0; i<kk.size(); i++) {
      filename << datapath_ << "/" << sord << "_alphas" << sas(_alphas[kk[i]]) << "_nf" << _nf << ".gamma.table";
      HTP[kk[i]] = new HellTableP();
      HTP[kk[i]]->readTable(filename.str());
      HTP[kk[i]]->setMellinFunction(DeltaGamma);
      filename.str("");  filename.clear();
    }
  }
  void HELLnf::ReadTable2(int k, string id) {
    ostringstream filename;
    vector<int> kk;
    for(int j=0; j<2; j++) {
      itHTP = HTP2[id].find(k+j);
      if(itHTP == HTP2[id].end()) kk.push_back(k+j);
    }
    //
    for(int i=0; i<kk.size(); i++) {
      filename << datapath_ << "/lobatnlo_alphas" << sas(_alphas2[kk[i]]) << "_nf" << _nf << ".gamma.table";
      //filename << datapath_ << "/nlo_alphas" << sas(_alphas2[kk[i]]) << "_nf" << _nf << ".gamma.table";   // hack!
      HTP2[id][kk[i]] = new HellTableP();
      HTP2[id][kk[i]]->readTable(filename.str());
      filename.str("");  filename.clear();
      //
      filename << datapath_ << "/lodot_alphas" << sas(_alphas2[kk[i]]) << "_nf" << _nf << ".gamma.table";
      HTP2[id][kk[i]]->readDotTable(filename.str());
      filename.str("");  filename.clear();
      //
      filename << datapath_ << "/nlo_alphas" << sas(_alphas2[kk[i]]) << "_nf" << _nf << ".gamma.table";
      if(id == "P+Uqg") HTP2[id][kk[i]]->readNLOTable(filename.str());
      filename.str("");  filename.clear();
      //
      if(id == "Pqg")    HTP2[id][kk[i]]->setMellinFunction(DeltaGammaResNLOqg);
      //if(id == "Pqg")    HTP2[id][kk[i]]->setMellinFunction(newDeltaGammaResNLOqg);
      if(id == "P+Uqg")  HTP2[id][kk[i]]->setMellinFunction(DeltaGammaPlusUqg);
      if(id == "C2")     HTP2[id][kk[i]]->setMellinFunction(DeltaC2Res);
      if(id == "CL")     HTP2[id][kk[i]]->setMellinFunction(DeltaCLRes);
      if(id == "oldCL")  HTP2[id][kk[i]]->setMellinFunction(oldDeltaCLRes);
      if(id == "oldC2")  HTP2[id][kk[i]]->setMellinFunction(oldDeltaC2Res);
    }
  }

  // Delta P_+
  double HELLnf::xdeltaPplus(double as, double x) {
    double factor;
    int k = alphas_interpolation(as, _alphas, factor);
    ReadTable(k);
    double dPf_k1 = HTP[k  ]->eval(x);
    double dPf_k2 = HTP[k+1]->eval(x);
    double res = dPf_k1 + factor * (dPf_k2 - dPf_k1);
    return res * pow(1-x,_damping);
  }
  // Delta P_qg
  double HELLnf::xdeltaPqg(double as, double x) {
    //return xdeltaC(as, x, "P+Uqg"); // hack to get Pgg from the RC rotation
    return xdeltaC(as, x, "Pqg");
  }
  double HELLnf::xdeltaPpUqg(double as, double x) {
    return xdeltaC(as, x, "P+Uqg");
  }
  // Delta C
  double HELLnf::xdeltaC(double as, double x, string id) {
    if(_order==0) return 0;
    double factor;
    int k = alphas_interpolation(as, _alphas2, factor);
    ReadTable2(k, id);
    //HTP2[id][k]->setErrorEstimation(true);
    double dPf_k1 = HTP2[id][k  ]->eval(x);
    double dPf_k2 = HTP2[id][k+1]->eval(x);
    //cout << "error="<<HTP2[id][k]->getError() << endl;
    double res = dPf_k1 + factor * (dPf_k2 - dPf_k1);
    return res * pow(1-x,_damping);
  }


  struct intvars {
    HELLnf* h;
    double as;
  };
  double gplintegrand(double t, void* p) {
    intvars vars = *(intvars *) p;
    HELLnf *hell = vars.h;
    double x = exp(-t);
    return x*hell->xdeltaPplus(vars.as,x);
  }
  double gqgintegrand(double t, void* p) {
    intvars vars = *(intvars *) p;
    HELLnf *hell = vars.h;
    double x = exp(-t);
    return x*hell->xdeltaPqg(vars.as,x);
  }
  double HELLnf::gpl2(double as) {
    itMC = MCpl.find(as);
    if (itMC == MCpl.end()) {
      double prec = 1e-4;
      intvars p = { this, as };
      double tmax = log(1.e6);
      MCpl[as] = integration_gauss(gplintegrand, 0, tmax, prec, NULL, &p);
    }
    return MCpl[as];
  }
  double HELLnf::gqg2(double as) {
    itMC = MCqg.find(as);
    if (itMC == MCqg.end()) {
      double prec = 1e-4;
      intvars p = { this, as };
      double tmax = log(1.e6);
      MCqg[as] = integration_gauss(gqgintegrand, 0, tmax, prec, NULL, &p);
    }
    return MCqg[as];
  }





  // Splitting Function Matrix
  sqmatrix<double> HELLnf::DeltaPLL(double x, double as, Order matched_to_fixed_order) {
    if(matched_to_fixed_order != LO) {
      cout << "Error: trying to match LL resummation in splitting functions to a fixed order different from LO" << endl;
      exit(45);
    }
    double dGgg = xdeltaPplus(as, x)/x;
    //if(matched_to_fixed_order < LO) dGgg += as*CA/M_PI;
    if(_momcons) {
      //dGgg -= 6*(1-x)*gpl2(as);
      dGgg -= 12*(1-x)*(1-x)*gpl2(as);
      //dGgg -= 20*(1-x)*(1-x)*(1-x)*gpl2(as);
    }
    // Both of the following options are equally acceptable
    return sqmatrix<double>(dGgg, CF/CA*dGgg, 0., 0.);
    //return sqmatrix<double>(dGgg, 0., 0., 0.);
  }
  sqmatrix<double> HELLnf::DeltaPNLL(double x, double as, Order matched_to_fixed_order) {
    if(matched_to_fixed_order != NLO) {
      cout << "Error: trying to match NLL resummation in splitting functions to a fixed order different from NLO" << endl;
      exit(45);
    }
    double dGp  = xdeltaPplus(as, x)/x;
    double dGqg = xdeltaPqg(as, x)/x;
    /*
    if(matched_to_fixed_order < NLO) {
      dGp  += as*as/M_PI/M_PI/4. * _nf*(26.*CF-23.*CA)/9./x;
      dGqg += as*as/M_PI/M_PI/4. * _nf*20.*CA/9./x;
    }
    if(matched_to_fixed_order < LO) {
      cout << "Error: cannot use pure NLL in x-space: delta(1-x) terms are missing" << endl;
      abort();
    }
    if(matched_to_fixed_order > NLO) {
      dGp  -= as*as*as/M_PI/M_PI/M_PI/8. * (-12.38818182*CA*CA*CA - 3.066013837*CA*CA*_nf + 6.132027674*CA*CF*_nf) * (-log(x)/x);
      dGqg -= as*as*as/M_PI/M_PI/M_PI/8. * 112.*CA*CA*_nf/27. * (-log(x)/x);
    }
    */
    double dGgg = dGp - CF/CA*dGqg;
    if(_momcons) {
      //dGgg -= 6*(1-x)* ( gpl2(as) + (1-CF/CA)*gqg2(as) );
      dGgg -= 12*(1-x)*(1-x)* ( gpl2(as) + (1-CF/CA)*gqg2(as) );
    }
    return sqmatrix<double>(dGgg, CF/CA*dGgg, dGqg, CF/CA*dGqg);
  }
  sqmatrix<double> HELLnf::DeltaP(double as, double x) {//, Order matched_to_fixed_order) {
    if(_order==0) {
      //return DeltaPLL(x, as, matched_to_fixed_order);
      return DeltaPLL(x, as, LO);
    } else if(_order==1) {
      //return DeltaPNLL(x, as, matched_to_fixed_order);
      return DeltaPNLL(x, as, NLO);
    }
    return sqmatrix<double>(0,0,0,0);
  }
  double HELLnf::deltaPplus(double as, double x) {
    return xdeltaPplus(as, x) /x;
  }
  // Delta Coefficient function
  double HELLnf::deltaC2g(double as, double x) {
    //sqmatrix<double> dP = DeltaP(as,x,NLO);
    return xdeltaC(as, x, "C2") /x;// - dP.qg()/dP.gg();
  }
  double HELLnf::deltaCLg(double as, double x) {
    return xdeltaC(as, x, "CL") /x;
  }
  double HELLnf::deltaC2gold(double as, double x) {
    return xdeltaC(as, x, "oldC2") /x;
  }
  double HELLnf::deltaCLgold(double as, double x) {
    return xdeltaC(as, x, "oldCL") /x;
  }





};

