/*#include "complex.hh"
#include <math.h>

static dcomplex coeff(dcomplex a, int n){
  dcomplex c=1.;
  for(int i=0; i<n; i++){
    c *= a+1.*i;
  }
  return c;
}

static dcomplex lzs(){
  
}
*/

//#include "dcomplex.hh"
//#include <math.h>

#define GSL_DBL_EPSILON 2.2204460492503131e-16
#define GSL_SQRT_DBL_EPSILON   1.4901161193847656e-08
#define INT_THRESHOLD (1000.0*GSL_DBL_EPSILON)
//#define M_SQRT2 1.41421356237309504880168872421

inline double ABS(dcomplex x) {return abs(x);}

/*
  2F0 asymptotics  (see eg Abramowitz, Stegun, 13.5.2, pag 508)
  NB: it's a divergent series, so it is treaten as an asymptotic series
*/
dcomplex chf_2F0_asympt(dcomplex a, dcomplex b, dcomplex z) {
  // previous step
  dcomplex Ap = 1.;
  dcomplex Sp = Ap;
  // current step
  dcomplex A = z*a*b;
  dcomplex S = Sp + A;
  //
  int MaxIter = 500;
  double tolerance = 1.e-10;
  for(double k=1; k<MaxIter; k++) {
    Ap = A;
    A *= (a+k)*(b+k) * z/(k+1.);
    if( ABS(A)/ABS(S) < tolerance && ABS(Ap)/ABS(Sp) < tolerance) return S+A;
    if( ABS(A)/ABS(Ap) > 1.) return S;
    Sp = S;
    S += A;
  }
  cout << "ERROR: 2F0(a,b,z) asymptotic did not provide a good approximation for a=" << a << " b=" << b << " z=" << z << endl;
  return S+A;
}





/* Large x asymptotic for  x^a U(a,b,x)
 * Based on SLATEC D9CHU() [W. Fullerton]
 *
 * Uses a rational approximation due to Luke.
 * See [Luke, Algorithms for the Computation of Special Functions, p. 252]
 *     [Luke, Utilitas Math. (1977)]
 *
 * z^a U(a,b,z) ~ 2F0(a,1+a-b,-1/z)
 *
 * This assumes that a is not a negative integer and
 * that 1+a-b is not a negative integer. If one of them
 * is, then the 2F0 actually terminates, the above
 * relation is an equality, and the sum should be
 * evaluated directly [see below].
 */

static dcomplex d9chu(const dcomplex a, const dcomplex b, const dcomplex x){

  const double EPS   = 8.0 * GSL_DBL_EPSILON;  /* EPS = 4.0D0*D1MACH(4)   */
  const int maxiter = 500;
  dcomplex aa[4], bb[4];
  int i;

  dcomplex bp = 1.0 + a - b;
  dcomplex ab = a*bp;
  dcomplex ct2 = 2.0 * (x - ab);
  dcomplex sab = a + bp;
  
  dcomplex ct3 = sab + 1.0 + ab;
  dcomplex anbn = ct3 + sab + 3.0;
  dcomplex ct1 = 1.0 + 2.0*x/anbn;

  bb[0] = 1.0;
  aa[0] = 1.0;

  bb[1] = 1.0 + 2.0*x/ct3;
  aa[1] = 1.0 + ct2/ct3;
  
  bb[2] = 1.0 + 6.0*ct1*x/ct3;
  aa[2] = 1.0 + 6.0*ab/anbn + 3.0*ct1*ct2/ct3;

  for(i=4; i<maxiter; i++) {
    int j;
    dcomplex c2;
    dcomplex d1z;
    dcomplex g1, g2, g3;
    dcomplex x2i1 = 2*i - 3;
    ct1   = x2i1/(x2i1-2.0);
    anbn += x2i1 + sab;
    ct2   = (x2i1 - 1.0)/anbn;
    c2    = x2i1*ct2 - 1.0;
    d1z   = 2.0*x2i1*x/anbn;
    
    ct3 = sab*ct2;
    g1  = d1z + ct1*(c2+ct3);
    g2  = d1z - c2;
    g3  = ct1*(1.0 - ct3 - 2.0*ct2);
    
    bb[3] = g1*bb[2] + g2*bb[1] + g3*bb[0];
    aa[3] = g1*aa[2] + g2*aa[1] + g3*aa[0];
    
    if( ABS(aa[3]*bb[0]-aa[0]*bb[3]) < EPS* ABS(bb[3]*bb[0])) break;
    
    for(j=0; j<3; j++) {
      aa[j] = aa[j+1];
      bb[j] = bb[j+1];
    }
  }
  
  if(i == maxiter) {
    cerr << "d9chu over maxiter" << endl;
  }
  return aa[3]/bb[3];
  
}


/* Evaluate asymptotic for z^a U(a,b,z) ~ 2F0(a,1+a-b,-1/z)
 * We check for termination of the 2F0 as a special case.
 * Assumes x > 0.
 * Also assumes a,b are not too large compared to x.
 */
static dcomplex hyperg_zaU_asymp(const dcomplex a, const dcomplex b, const dcomplex x){

  const dcomplex ap = a;
  const dcomplex bp = 1.0 + a - b;
  const double rintap = floor(real(ap + 0.5));
  const double rintbp = floor(real(bp + 0.5));
  const int ap_neg_int = ( fabs(imag(ap)) < INT_THRESHOLD && real(ap) < 0.0 && fabs(real(ap - rintap)) < INT_THRESHOLD );
  const int bp_neg_int = ( fabs(imag(bp)) < INT_THRESHOLD && real(bp) < 0.0 && fabs(real(bp - rintbp)) < INT_THRESHOLD );

  if(ap_neg_int || bp_neg_int) {
    /* Evaluate 2F0 polynomial. */
    dcomplex mxi = -1.0/x;
    //double nmax = -(int)(MIN(real(ap),real(bp)) - 0.1); // I'm not sure it's correct for dcomplex.... (nor for double..)
    int nmax = -(int)(ap_neg_int ? rintap : rintbp); // This seems correct to me.
    dcomplex tn  = 1.0;
    dcomplex sum = 1.0;
    double sum_err = 0.0;
    for(double n=1; n <= nmax; n++) {
      dcomplex apn = (ap+n-1.0);
      dcomplex bpn = (bp+n-1.0);
      tn  *= ((apn/n)*mxi)*bpn;
      sum += tn;
      sum_err += 2.0 * GSL_DBL_EPSILON * ABS(tn);
      n += 1;
    }
    return sum;
  }
  else {
    //return chf_2F0_asympt(a, bp, -1./x);  // quite good as well, but not enough accurate
    return d9chu(a,b,x);
  }
}

//#include "CONHYP/conhyp.hh"
//#include "hyperg1F1.cc"
dcomplex hypergU(dcomplex a, dcomplex b, dcomplex z) {
  /*
  double rintb   = floor(real(b + 0.5));
  bool b_zero_or_neg_int = ( fabs(imag(b)) < INT_THRESHOLD && real(b) < 0.5 && fabs(real(b - rintb)) < INT_THRESHOLD );
  if(b_zero_or_neg_int) return hypergU(1.+a-b, 2.-b, z) * pow(z,1-b);
  */
  //return chf_U(a, b, z);
  return pow(z,-a)*hyperg_zaU_asymp(a, b, z);
  //return conhyp(a, b, z); // it's 1F1 !!!!! wrong!!!
}




dcomplex inverse_coefficient(dcomplex *coeff_ser, dcomplex *coeff_inv_ser, int k) {
  dcomplex res = 0;
  for(int i=1; i<=k; i++) res -= coeff_ser[i] * coeff_inv_ser[k-i];
  return res/coeff_inv_ser[0];
}
dcomplex multiplication_coefficient(dcomplex *coeff_ser_1, dcomplex *coeff_ser_2, int k) {
  dcomplex res = 0;
  for(int i=0; i<=k; i++) res += coeff_ser_1[i] * coeff_ser_2[k-i];
  return res;
}
dcomplex aux_Bateman_logderiv_asympt(dcomplex a, dcomplex z) {
  dcomplex mzi = -1./z;
  cout << "-1/z = " << mzi << endl;
  const int MaxIter = 500;
  double tolerance = 1.e-10;
  dcomplex Ak[MaxIter];
  dcomplex Bk[MaxIter];
  dcomplex iBk[MaxIter];
  dcomplex Ck[MaxIter];
  dcomplex S[MaxIter];
  Ak[0]  = 1.;
  Bk[0]  = 1.;
  iBk[0] = 1.;
  Ck[0]  = 1.;
  S[0]   = 1.;
  for(int k=1; k<MaxIter; k++) {
    double kk = k;
    Ak[k]  = Ak[k-1] * (a+1.+kk)*(a+1.+kk) / (kk+1.);
    Bk[k]  = Bk[k-1] * (a+1.+kk)*(a   +kk) / (kk+1.);
    iBk[k] = inverse_coefficient(Bk, iBk, k);
    Ck[k]  = multiplication_coefficient(Ak, iBk, k);
    S[k]   = S[k-1] + Ck[k]*mzi;

    cout << k
	 << "   " << Ak[k]
	 << "   " << Bk[k]
	 << "   " << iBk[k]
	 << "   " << Ck[k]
	 << "   " << S[k]
	 << endl;

    if(k>1) if( ABS(Ck[k]*pow(mzi,k-1))/ABS(S[k-1]) < tolerance && ABS(Ck[k-1]*pow(mzi,k-1))/ABS(S[k-2]) < tolerance) return S[k];
    if( ABS(Ck[k])/ABS(Ck[k-1]) > ABS(z)) return S[k-1];
  }
  cout << "ERROR: Bateman_log_deriv asymptotic did not provide a good approximation for a=" << a << " z=" << z << endl;
  return S[MaxIter-1];
}
dcomplex Bateman_logderiv_asympt(dcomplex nu, dcomplex x) {
  dcomplex res = -1. + nu * aux_Bateman_logderiv_asympt(-nu*0.5,2.*x) /2./x;
  return res;
}
