#include "math/pade_approximant.hh"
//#include <ginac/ginac.h>
#include <gsl/gsl_linalg.h>
#include <gsl/gsl_complex_math.h>

using namespace std;


/*
void __GinacPadeCoefficients(const GiNaC::numeric *coeff, const unsigned int n, GiNaC::numeric *a, GiNaC::numeric *b) {
  using namespace GiNaC;
  matrix A(n,n);
  matrix rhs(n,1);
  symbol x[n];
  matrix var(n,1);
  for(int i=0; i<n; i++) {
    for(int j=0; j<n; j++)
      A(i, j) = coeff[n-j+i];
    rhs(i,0) = -coeff[n+i+1];
    var(i,0) = x[i];
  }
  matrix sol(n,1);
  sol = A.solve(var,rhs);
  for(int i=0; i<n; i++)
    b[i] = ex_to<numeric>(sol(i,0));
  for(int i=0; i<=n; i++) {
    a[i] = coeff[i];
    for(int j=0; j<i; j++)
      a[i] += coeff[j]*b[i-j-1];
  }
}

void ginacPadeCoefficients(dcomplex *coeff, const unsigned int n, dcomplex *a, dcomplex *b) {
  GiNaC::numeric *aa = new GiNaC::numeric[n+1];
  GiNaC::numeric *bb = new GiNaC::numeric[n];
  GiNaC::numeric *cc = new GiNaC::numeric[2*n+1];
  for(int i=0; i<=2*n; i++)
    cc[i] = real(coeff[i]) + GiNaC::I * imag(coeff[i]);
  __GinacPadeCoefficients(cc, n, aa, bb);
  a[0] = GiNaC::real(aa[0]).to_double() + I*GiNaC::imag(aa[0]).to_double();
  for(int i=0; i<n; i++) {
    a[i+1] = GiNaC::real(aa[i+1]).to_double() + I*GiNaC::imag(aa[i+1]).to_double();
    b[i]   = GiNaC::real(bb[i]  ).to_double() + I*GiNaC::imag(bb[i]  ).to_double();
  }
}
*/

inline gsl_complex cmplx_to_gsl(dcomplex x) {
  return gsl_complex_rect(real(x), imag(x));
}
inline dcomplex gsl_to_cmplx(gsl_complex x) {
  return dcomplex(GSL_REAL(x), GSL_IMAG(x));
}
// [n/n]
void gslPadeCoefficients(dcomplex *coeff, const unsigned int n, dcomplex *a, dcomplex *b) {
  gsl_vector_complex *sol = gsl_vector_complex_alloc(n);
  gsl_vector_complex *rhs = gsl_vector_complex_alloc(n);
  gsl_matrix_complex *A   = gsl_matrix_complex_alloc(n,n);
  for(int i=0; i<n; i++) {
    for(int j=0; j<n; j++)
      gsl_matrix_complex_set(A, i,j, cmplx_to_gsl(coeff[n-j+i]));
    gsl_vector_complex_set(rhs, i, cmplx_to_gsl(-coeff[n+i+1]));
  }
  int s;
  gsl_permutation *p = gsl_permutation_alloc(n);
  gsl_linalg_complex_LU_decomp(A,p,&s);
  gsl_linalg_complex_LU_solve (A,p,rhs,sol);
  for(int i=0; i<n; i++) {
    gsl_complex bb = gsl_vector_complex_get(sol,i);
    b[i] = gsl_to_cmplx(bb);
  }
  for(int i=0; i<=n; i++) {
    a[i] = coeff[i];
    for(int j=0; j<i; j++)
      a[i] += coeff[j]*b[i-j-1];
  }
  gsl_permutation_free(p);
  gsl_vector_complex_free(sol);
  gsl_vector_complex_free(rhs);
  gsl_matrix_complex_free(A);
}
void PadeCoefficients(dcomplex *coeff, const unsigned int n, dcomplex *a, dcomplex *b) {
  //ginacPadeCoefficients(coeff,n,a,b);
  gslPadeCoefficients(coeff,n,a,b);
}


dcomplex PadeApproximant(dcomplex z, const dcomplex *a, const dcomplex *b, const unsigned int n) {
  dcomplex num = a[0], den = 1.;
  for(int i=1; i<=n; i++) {
    num += a[i]   * pow(z, i);
    den += b[i-1] * pow(z, i);
  }
  return num/den;
}



// [n/m]
void gslPadeCoefficients(dcomplex *coeff, const unsigned int n, const unsigned int m, dcomplex *a, dcomplex *b) {
  gsl_vector_complex *sol = gsl_vector_complex_alloc(m);
  gsl_vector_complex *rhs = gsl_vector_complex_alloc(m);
  gsl_matrix_complex *A   = gsl_matrix_complex_alloc(m,m);
  for(int i=0; i<m; i++) {
    gsl_vector_complex_set(rhs, i, cmplx_to_gsl(-coeff[n+1+i]));
    for(int j=0; j<m; j++)
      gsl_matrix_complex_set(A, i,j, (n+i<j ? cmplx_to_gsl(0) : cmplx_to_gsl(coeff[n-j+i])));
  }
  int s;
  gsl_permutation *p = gsl_permutation_alloc(m);
  gsl_linalg_complex_LU_decomp(A,p,&s);
  gsl_linalg_complex_LU_solve (A,p,rhs,sol);
  for(int i=0; i<m; i++) {
    gsl_complex bb = gsl_vector_complex_get(sol,i);
    b[i] = gsl_to_cmplx(bb);
  }
  for(int i=0; i<=n; i++) {
    a[i] = coeff[i];
    for(int j=0; j<i; j++)
      if(i<=m+j) a[i] += coeff[j]*b[i-j-1];
  }
  gsl_permutation_free(p);
  gsl_vector_complex_free(sol);
  gsl_vector_complex_free(rhs);
  gsl_matrix_complex_free(A);
}
void PadeCoefficients(dcomplex *coeff, const unsigned int n, const unsigned int m, dcomplex *a, dcomplex *b) {
  gslPadeCoefficients(coeff,n,m,a,b);
}
dcomplex PadeApproximant(dcomplex z, const dcomplex *a, const dcomplex *b, const unsigned int n, const unsigned int m) {
  dcomplex num = a[0], den = 1.;
  for(int i=1; i<=n; i++)
    num += a[i]   * pow(z, i);
  for(int i=1; i<=m; i++)
    den += b[i-1] * pow(z, i);
  return num/den;
}


